#!/bin/bash

#DESCRIPTION: switch ON/OFF program and console state for /dev/ttyS*
# main code get from Gumer T. 2016 gumer@mcst.ru
# rewrite Alexey Mukhin if@mcst.ru 2017

OPTIONS="hd:RSV"

FUSER=`which fuser`

#FLAG == 1 -- run, else stop
FLAG=1
DEVICE="/dev/ttyS0"
VERBOSE=0

while getopts $OPTIONS OPT
do
	case $OPT in
	R ) FLAG=1;; #default
	S ) FLAG=0;;
	V ) VERBOSE=1;;
	d ) DEVICE="${OPTARG}";;
	h ) echo "
 Usage: $0 [-R|-S] -d /dev/ttyS0
	-R - run system (OS) work with device -- default
	-S - stop system (OS) work with device
	-V - show messages for user
	-d - path to device -- default: \"/dev/ttyS0\"
";exit 0;;
	esac
done

check_dev_tty () {
	Dev=$1; Dev=${Dev#/dev/}
	#     /proc/console_ctrl
	IS_CNSL_CTRL=`[[ -f /proc/console_ctrl ]];\
					( (( ! $? )) && echo 1 ) || echo 0`
	PID_ACT_COM=`${FUSER} $1 2> /dev/null`
	(( $VERBOSE )) && echo "pid list $PID_ACT_COM"
	if (( $IS_CNSL_CTRL )); then
		IS_CMD_LINE=`cat /proc/console_ctrl | grep $Dev | grep able`
		[[ ! -z $IS_CMD_LINE ]] && {\
			read N_TTY TTY DE <<< $IS_CMD_LINE;\
			N_TTY=${N_TTY%:}; TTY=${TTY%:}; }
	else
#		[[ -z $2 ]] && return
		CL=(`cat /proc/cmdline`)
		K=0
		while [[ ! -z ${CL[$K]} ]]
		do
			(echo ${CL[$K]} | grep console | grep $Dev) > /dev/null
			(( $VERBOSE )) && (( ! $? )) && {\
				echo -e "!!! Warning !!!"\
				"Not found /proc/console_ctrl.\n"\
				"\tIn cmdline found\t ${CL[$K]} and\n"\
				"\tcheck device\t=$Dev.\n"\
				"\tMay be errors.";\
				break; }
			((K++))
		done
	fi
}

activate_tty () {
	CMD_kill=STOP; ACT=-; M_ACT_A="STOP"; M_ACT_T="BLOCKED"
	[[ $2 == "1" ]] && {\
		CMD_kill=CONT; ACT=+; M_ACT_A="RUN";\
					M_ACT_T="RUNING"; }
	check_dev_tty $1
	(( $VERBOSE )) && echo -e "State $1 :\n"\
		"\tIS_CNSL_CTRL =$IS_CNSL_CTRL"\
		"\n\tPID_ACT_COM =$PID_ACT_COM"\
		"\n\tconsole\t=$DE"

	if [ "$PID_ACT_COM" != "" ]; then
		for p in $PID_ACT_COM
		do
			if [[ ! -d /proc/$p ]]; then
				(( $VERBOSE )) && echo -e "process '$p' not exists"
				continue;
			fi
			read NmSt St0 <<< $(cat /proc/$p/status 2>/dev/null | grep State)
			kill -s $CMD_kill $p
			read NmSt St <<< $(cat /proc/$p/status 2>/dev/null | grep State)
			PROG=`cat /proc/${p}/comm 2>/dev/null`
			(( $VERBOSE )) && echo -e "$M_ACT_A process '$PROG' on $Dev"\
				"\n\tN proc = $p :$NmSt $St0 -> $St"
		done
	fi
	if [[ ! -z $N_TTY ]]; then
		echo "$ACT${N_TTY}" > /proc/console_ctrl
		read N_TTY TTY DE <<<\
			`cat /proc/console_ctrl | grep $Dev | grep able`
		(( $VERBOSE )) && echo -e "Output OS on $TTY $M_ACT_T (Status :$DE)"
	fi
}

(( $VERBOSE )) && echo "Device: $DEVICE"
if (( $VERBOSE )); then
   (( $FLAG )) && echo "Flag: RUN"
   (( $FLAG )) || echo "Flag: STOP"
fi

activate_tty $DEVICE $FLAG

exit 0
