#!/bin/bash

#DESCRIPTION: create /dev/dsp* and /dev/mixer* devices if not present
# main code get from Gumer T. 2016 gumer@mcst.ru
# rewrite Alexey Mukhin 2017 if@mcst.ru

OPTIONS="hd:RSV"

LSPCI=`which lspci`
GREP=`which grep`
AWK=`which awk`
RMMOD=`which rmmod`
MODPROBE=`which modprobe`
MKNOD=`which mknod`
RM=`which rm`
LN=`which ln`

FLAG=0
#DEVICE="/dev/dsp0"
VERBOSE=0

while getopts $OPTIONS OPT
do
	case $OPT in
	R ) FLAG=1;;
	L ) FLAG=0;; #default
	V ) VERBOSE=1;;
#	d ) DEVICE="${OPTARG}";;
	h ) echo "
 Usage: $0 [-R|-L] [-d /dev/dsp0]
	-R - reset sound system (remove module then load)
	-L - load attempt sound system -- default
	-V - show messages for user
";exit 0;;
	esac
done

BUS_AUDIO=
MSG_AUDIO=
MOD_AUDIO=
MOD_LOADS=

search_sound_device()
{
BUS_AUDIO=(`lspci -vk | grep -i audio | grep -v Radeon | awk '{print $1}'`)
COUNT_AUDIO=(`lspci -vk | grep -i audio | grep -v Radeon | awk '{print $1}' | wc -l`)
DEC_COUNT=0
(( DEC_COUNT = $COUNT_AUDIO - 1 ))
MSG_AUDIO=`lspci -vk -s $BUS_AUDIO`
MOD_AUDIO=`echo -e "$MSG_AUDIO" | grep "Kernel modules" | awk '{print $3}'`
MOD_LOADS=`echo -e "$MSG_AUDIO" | grep "Kernel driver in use" | awk '{print $5}'`
}

remove_sound_module()
{
    (( $VERBOSE )) && echo -e "Remove module:  $MOD_AUDIO"
    $RMMOD $MOD_AUDIO 2>/dev/null
    (( $VERBOSE )) && echo -e "Remove files:   /dev/dsp[0-3]"
    $RM -rf /dev/dsp > /dev/null
    $RM -rf /dev/dsp[0-3] > /dev/null
    (( $VERBOSE )) && echo -e "Remove files:   /dev/mixer[0-3]"
    $RM -rf /dev/mixer > /dev/null
    $RM -rf /dev/mixer[0-3] > /dev/null
}

load_sound_module()
{
    if [[ ! -z $MOD_LOADS ]]; then
	#hack for new systems  (as Elbrus-D)
	$MODPROBE snd_pcm_oss 2>/dev/null
	$MODPROBE snd_mixer_oss 2>/dev/null
	return
    fi
    (( $VERBOSE )) && echo -e "Load module:    $MOD_AUDIO"
    $MODPROBE $MOD_AUDIO 2>/dev/null
    #hack for new systems (as Elbrus-D)
    $MODPROBE snd_pcm_oss 2>/dev/null
    $MODPROBE snd_mixer_oss 2>/dev/null
}

create_sound_device()
{
    (( $VERBOSE )) && echo -e "Found $COUNT_AUDIO sound devices"

    if [[ ! -e /dev/dsp ]]; then
	(( $VERBOSE )) && echo -e "Remove files:     /dev/dsp /dev/dsp0"
	$RM -rf /dev/dsp /dev/dsp0 > /dev/null
	(( $VERBOSE )) && echo -e "Create dev-files: /dev/dsp /dev/dsp0"
	$MKNOD /dev/dsp c 14 3
	cd /dev/
	$LN -s dsp dsp0
    else
	if [[ ! -e /dev/dsp0 ]]; then
	    cd /dev/
	    $LN -s dsp dsp0
	fi
    fi

    for i in `seq 1 $DEC_COUNT`
    do
	if [[ ! -e /dev/dsp$i ]]; then
	    (( $VERBOSE )) && echo -e "Create file:   /dev/dsp$i"
	    (( MINOR = 16 * $i + 3 ))
	    $MKNOD /dev/dsp$i c 14 $MINOR
	fi
    done

    if [[ ! -e /dev/mixer ]]; then
	(( $VERBOSE )) && echo -e "Remove files:     /dev/mixer /dev/mixer0"
	$RM -rf /dev/mixer /dev/mixer0 > /dev/null
	(( $VERBOSE )) && echo -e "Create dev-files: /dev/mixer /dev/mixer0"
	$MKNOD /dev/mixer c 14 0
	cd /dev/
	$LN -s mixer mixer0
    else
	if [[ ! -e /dev/mixer0 ]]; then
	    cd /dev/
	    $LN -s mixer mixer0
	fi
    fi

    for i in `seq 1 $DEC_COUNT`
    do
	if [[ ! -e /dev/mixer$i ]]; then
	    (( $VERBOSE )) && echo -e "Create file:   /dev/mixer$i"
	    $RM -rf /dev/mixer$i > /dev/null
	    (( MINOR = 16 * $i ))
	    $MKNOD /dev/mixer$i c 14 $MINOR
	fi
    done
}

search_sound_device

#(( $VERBOSE )) && echo "Device: $DEVICE"
if (( $VERBOSE )); then
   (( $FLAG )) && echo "Flag: RESET anyway"
   (( $FLAG )) || echo "Flag: LOAD if needed"
   echo "Module for dev: $MOD_AUDIO"
   echo "Module  in use: $MOD_LOADS"
fi

(( $FLAG )) && remove_sound_module
load_sound_module
create_sound_device

exit 0
