#!/bin/bash

# set_route :
#   ethernet setup for fake loopback interface with NAT usage
#   Let ETH_1 and ETH_2 be the two ethernet cards;
#   IP_1 and IP_2 its IP address;
#   MAC1 and MAC2 its MAC address respectively.
#   We will be using two dummy IP addresses: IPw_1 and IPw_2.

PSTP="/opt/mcst/check/STP/bin"
if [[ ! -d ${PSTP} ]]; then
	PSTP="/opt/mcst/STP/bin"
fi

umask 002

TMP="/tmp/STP"
[[ -d ${TMP} ]] || mkdir -m 0755 ${TMP}
IPTABLES="${TMP}/iptables.cfg"
IPROUTES="${TMP}/iproutes.cfg"
ARP="${TMP}/arp.cfg"
MSG="${TMP}/msg-err"

set_route ()
{
	((N_in_set_route++))
	((VRB)) && echo -e "\nset_route $*"
	ETH_1=$1; IP_1=$2; MAC1=$3; IPw_1=$4
	ETH_2=$5; IP_2=$6; MAC2=$7; IPw_2=$8
	{
	ip route add $IPw_1 dev $ETH_2
	ip route add $IPw_2 dev $ETH_1

	iptables -t nat -A POSTROUTING -d $IPw_2 -j SNAT --to-source $IPw_1
	iptables -t nat -A POSTROUTING -d $IPw_1 -j SNAT --to-source $IPw_2
	iptables -t nat -A PREROUTING -d $IPw_1 -j DNAT --to-destination $IP_1
	iptables -t nat -A PREROUTING -d $IPw_2 -j DNAT --to-destination $IP_2

	arp -s $IPw_1 $MAC1
	arp -s $IPw_2 $MAC2
	} 2>&1 | grep "initialize iptables table" > ${MSG}
} # END ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ set_route

up ()
{
	PRINT=1
	MINIMAL=0
	[[ $1 == "minimal" ]] && MINIMAL=1
	if [[ $1 == "silent" ]]; then
		PRINT=0;MINIMAL=0
	fi
	iptables-save > ${IPTABLES}
	ip route save scope link > ${IPROUTES}

	IPs=()
	MACs=()
	ETHs=()
	IPWs=()
	for e in /sys/class/net/eth*
	do
		e=${e#$"/sys/class/net/"}

		ip=`ifconfig $e |grep "inet addr"`
		ip=${ip#*addr:}
		ip=${ip% Bcast*}

		mac=`ifconfig $e |grep "HWaddr"|awk '{print $5}'`

		ETHs+=($e)
		IPs+=($ip)
		MACs+=($mac)
	done

	c=1
	for ((i=0; i < (( ${#IPs[@]} - 1 )); ++i))
	do
		for ((j=$i + 1; j < ${#IPs[@]}; ++j))
		do
			IPWs+=("2.2.${c}.${i}")
			IPWs+=("2.2.${c}.${j}")
			set_route ${ETHs[i]} ${IPs[i]} ${MACs[i]} 2.2.${c}.${i} ${ETHs[j]} ${IPs[j]} ${MACs[j]} 2.2.${c}.${j}

			ping_res=0
			#ping -c 1 2.2.${c}.${i} > /dev/null
			${PSTP}/fastping -s 2.2.${c}.${i} > /dev/null
			((ping_res+=$?))
			#ping -c 1 2.2.${c}.${j} > /dev/null
			${PSTP}/fastping -s 2.2.${c}.${j} > /dev/null
			((ping_res+=$?))

			if [ ${ping_res} -eq 0 ]; then
				link="---"
				st="ok"
				(($MINIMAL)) && PRINT=1
			else
				link="-x-"
				st="no link"
				(($MINIMAL)) && PRINT=0
			fi

			(($PRINT)) && printf "%s\n"  "${ETHs[i]} ${IPs[i]} 2.2.${c}.${i} ${link} ${ETHs[j]} ${IPs[j]} 2.2.${c}.${j} ${st}"


			((c++))
		done
		if (($MINIMAL)) || (($SILENT)); then
			echo
		fi
	done

	printf "%s\n" "${IPWs[@]}" > ${ARP}
}

down()
{
	[[ -f ${ARP} ]] || return

	ips=$(<${ARP})

	for i in $ips
	do
		arp -d $i
	done

	iptables-restore < ${IPTABLES}
	ip route flush scope link
	ip route restore < ${IPROUTES}
}

usage()
{ IFS=''; while read helpstr
do
	echo "$helpstr"
done << EOF
Setups network adapters for testing
./eth.sh        -- configures adapters and print only link
./eth.sh up     -- configures adapters
./eth.sh silent -- configures adapters and no print
./eth.sh down   -- restores settings
./eth.sh help   -- help
EOF
exit 1
}

args=("$@")
if (( $# )); then
	if [[ $1 == $0 ]]; then
		shift
		args=("$@")
	fi
fi

if [ "$#" -eq 0 ]
then
	up minimal
elif [ "$#" -eq 1 ]
then
	if [ ${args} = "up" ]; then
		up
	elif [ ${args} = "silent" ]; then
		up silent
	elif [ ${args} = "down" ]; then
		down
	elif [ ${args} = "help" ]; then
		usage
	else
		usage
	fi
else
	usage
fi

exit 0
