#!/usr/bin/env python

import socket
import sys
import os
import optparse
import json

prog_dir = os.path.dirname(os.path.abspath(__file__))
stp_dir = os.path.abspath(os.path.join(prog_dir, os.path.pardir))
lib_dir = os.path.join(stp_dir, 'libpy/stp_gui')
sys.path.insert(0, lib_dir)

import State

MONITORNAME = '/tmp/STP/stp_monitor.dump'
KERNELNAME = '/tmp/STP/stp_kernel.dump'
IPADDR = '127.0.0.1'
PORT = 3425
filename = MONITORNAME
COMMAND = None
VKNAME = None
VERBOSE = False
HUMAN = False

# presentYAML = False
# YAML = False
# try:
#     import yaml
# except ImportError:
#     prog_dir = os.path.dirname(os.path.abspath(__file__))
#     sys.path.insert(0,  prog_dir + '/../libpy/lib/')
#     try:
#         import yaml
#         presentYAML = True
#     except ImportError:
#         pass

vkcmdlist = {'ON':('MDF_VK_ON', 'select VK for test'),
             'OFF':('MDF_VK_OFF', 'unselect VK for test'),
             'TMON':('WRK_TERMMONITOR', 'terminate monitor on target VK')}

vcmd = []
for i in vkcmdlist:
    vcmd.append(i)

cmdlist = {'START':('CMD_RUN', 'run testing'),
           'STOP':('CMD_STOP', 'stop testing'),
           'STATUS':('GET_MES_STATUS', 'get status from kernel (attempt -H option)'),
           'PAUSE':('CMD_PAUSE', 'pause testing, if tests running'),
           'CONNECT':('WRK_RECONNECT', 'attempt to connect for all disconnected VK'),
           'CONT':('CMD_CONTINUE', 'continue testing, if tests running before'),
           'RESET':('CMD_RESET', 'reset test results'),
           'TKERN':('WRK_TERMINATE', 'terminate kernel'),
           'TERMALL':('WRK_TERMALL', 'terminate all: kernel, monitors, but not others UI')}

cmdlist.update(vkcmdlist)

lcmd = []
for i in cmdlist:
    lcmd.append(i)
lcmd.sort()

def send_dump_cmd(ipadr, port, filename):
    print 'Address: ' + ipadr
    print 'Port: ' + str(port)
    print 'Dump in: ' + filename
    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    s = ('a' * 35)
    r = sock.sendto(s, (ipadr, port))
    if r != len(s):
        print 'Send error'
    sock.close()

def process_cmd_line():
    """Parse the command line and execute given commands."""
    parser = optparse.OptionParser("usage: %prog [options]")
    parser.add_option("-a", "--addr", action="store",
                      help="ip address for send cmd")
    parser.add_option("-c", "--command", action="store",
                      help="send command to kernel: " + ' '.join(lcmd) + ' ' + ''.join('HELP'))
    parser.add_option("-n", "--vkname", action="store",
                      help="name VK for commands: " + ' '.join(vcmd))
    parser.add_option("-p", "--port", action="store",
                      help="UDP port for send cmd")
    parser.add_option("-m", "--monitor", action="store_true",
                      help="send cmd to monitor")
    parser.add_option("-k", "--kernel", action="store_true",
                      help="send cmd to kernel")
    parser.add_option("-s", "--silent", action="store_true",
                      help="Silent output")
    parser.add_option("-v", "--verbose", action="store_true",
                      help="Show answer data (if present)")
    # parser.add_option("-y", "--yaml", action="store_true",
    #                   help="Show answer data in YAML mode (default JSON)")
    parser.add_option("-H", "--human", action="store_true",
                      help="Show data in answer only human mode (default: off)")
    (options, args) = parser.parse_args()

    global IPADDR
    global COMMAND
    global PORT
    global filename
    global VKNAME
    global VERBOSE
    # global YAML
    global HUMAN

    if options.silent:
        f = open(os.devnull, 'w')
	sys.stdout = f

    if options.verbose:
	VERBOSE = True

    # if options.yaml:
    #     if presentYAML:
    #         YAML = True
    #     else:
    #         print 'System: YAML module not found'

    if options.human:
        HUMAN = True

    if options.addr:
        IPADDR = options.addr

    if options.command:
        PORT = 9876
        filename = KERNELNAME
        COMMAND = options.command

    if options.vkname:
        VKNAME = options.vkname

    if options.port:
        PORT = int(options.port)

    if options.monitor:
        filename = MONITORNAME
        PORT = 3425

    if options.kernel:
        filename = KERNELNAME
        PORT = 9876
    return parser


def chunks(l, n):
    """
    Yield successive n-sized chunks from l.
    :param l:
    :param n:
    """
    for i in xrange(0, len(l), n):
        yield l[i:i+n]

CFG_CRC = 0

def create_message(type, data=None, answer=None, length=131072):
    """
    Create list with message-sequence. For simple message list contain one element
    :param type: message_type
    :param vkname: vk name
    :param data: databody
    :param answer: message_answer
    :param length: max length for message (split message)
    """
    m_type = (lambda T: " " + T if T else "")
    m_answer = (lambda A: " " + A + "\n" if A else "\n")
    m_crc = (lambda C: "CRC: " + str(C) + "\n" if C else "")
    m_size = (lambda D: "Size: " + str(D) + "\n" if D else "")
    m_multi = (lambda M: "Multiple: " + str(M) + "\n" if M else "")
    m_number = (lambda M: "Multi-Number: " + str(M) + "\n" if M else "")
    g_message = (lambda t, a, s, c, d: str("STP" + t + a + s + c + "\n" + d))
    g_multi = (lambda t, a, s, c, m, n, d: str("STP" + t + a + s + c + m + n + "\n" + d))

    global CFG_CRC

    messagelist = []
    if data:
        datalist = list(chunks(data, length))
        m = len(datalist)
        if m > 1:
            for i, d in enumerate(datalist):
                crc = CFG_CRC
                messagelist.append(g_multi(m_type(type), m_answer(answer),
                                            m_size(len(d)), m_crc(crc),
                                            m_multi(m), m_number(i+1), d))
        else:
            crc = CFG_CRC
            messagelist = [g_message(m_type(type), m_answer(answer),
                                      m_size(len(data)), m_crc(crc), data)]
    else:
        crc = 0
        messagelist = [g_message(m_type(type), m_answer(answer),
                                  m_size(None), m_crc(crc), "")]

    return messagelist


def simple_parse(message):
    global CFG_CRC
    separator = '\n'
    message = message.replace('\x00', '\r\n\r\n')
    message = message.replace('\r\n', separator)
    # split all data to head-body-head-body-...
    msg_st = list(message.split(separator * 2))
    meslist = []
    if not msg_st:
        return meslist

    i = 0
    size = 0
    while i < len(msg_st):
        part = msg_st[i]

        d = part.split(" ")
        if d[0] != 'STP':
            i += 2
            continue

        head = part.split(separator)
        for h in head[1:]:
            k, v = h.split(':')
            k = k.strip()
            v = v.strip()
            if k == 'Size':
                size = int(v)
            if k == 'CRC':
                CFG_CRC = v
                break;

        meslist.append((head[0], msg_st[i+1]))
        i += 2

    return (meslist, size)


def printStatus(stateString):
    st = State.State(0)
    j = json.loads(stateString)
    H = j.get('hosts', None)
    if H:
        for h in H:
            print h['hostname'] + ': ' + st.getTextStatus(' ', int(h['state']))
            T = h.get('tests', None)
            if T:
                for t in T:
                    print '  %-12s: %3d %3d %3d %3d\t%s' % (t['testname'], int(t['loop']), int(t['time']), int(t['passes']), int(t['errors']), st.getTextStatus(' ', int(t['state'])))

def cmd_transfer(sock, cmd, mess, info):
    ret = False
    m = create_message(cmd, mess)
    sock.send(m[0])

    a = ''
    count = 5
    while count:
        count -= 1
        try:
            a += sock.recv(1000000)
        except socket.error:
            print info + 'receive error'
            return False
        (s, size) = simple_parse(a)
        for i in s:
            # ignore MES_MESSAGE - only for GUI info
            if i[0].find('MES_MESSAGE') != -1:
                continue
            if i[0].find(cmd) != -1:
                if size-1 != len(str(i[1])):
                    break
                if VERBOSE and len(str(i[1])):
                    print 'Answer: '
                    # if YAML:
                    #     j = json.loads(str(i[1]))
                    #     print yaml.safe_dump(j, encoding='utf-8', indent=4, allow_unicode=True)
                    # else:
                    print str(i[1])

                if i[0].find('RPL_OK') != -1:
                    print info + 'successful'

                    if HUMAN and len(str(i[1])):
                        printStatus(str(i[1]))

                    return True
                count = 0
                break
    print info + 'command error'
    return False


def send_cmd(ipadr, port, cmd, vkname=None):
    message = None
    if vkname:
        message = '{"vkname":"' + str(vkname) + '"}'
    sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    try:
        sock.connect((ipadr, port))
    except socket.error:
        print 'Connect: error'
        return 1

    if not cmd_transfer(sock, 'MES_ACKNOWLEDGE', '1099999', 'Connect: '):
            sock.close()
            return 1

    if not cmd_transfer(sock, cmd, message, 'Execute: '):
            sock.close()
            return 1

    sock.close()
    return 0


def main():
    parser = process_cmd_line()
    if len(sys.argv) <= 1:
        parser.print_help()
        exit (1)
    if COMMAND:
        if not COMMAND in cmdlist:
            if COMMAND != 'HELP':
                print "Error:   unknown command - '" + COMMAND + "'."
            print "You may used next commands:"
            for i in lcmd:
                if i in vcmd:
                    print '\t- %-12s %s. Use it with "-n" option;' % (i, cmdlist[i][1])
                else:
                    print '\t- %-12s %s;' % (i, cmdlist[i][1])
            exit (2)
        print 'Address: ' + IPADDR
        print 'Port:    ' + str(PORT)

        if not COMMAND in vkcmdlist:
            global VKNAME
            VKNAME = None
        if COMMAND in vkcmdlist and not VKNAME:
            print 'Command: %-12s (%s. Use it with "-n" option)' % (COMMAND, cmdlist[COMMAND][1])
            print 'Error:   vkname empty'
            exit (3)
        print 'Command: %-12s (%s)' % (COMMAND, cmdlist[COMMAND][1])
        if VKNAME:
            print 'Message: ' + VKNAME
	r = send_cmd(IPADDR, PORT, cmdlist[COMMAND][0], VKNAME)
        exit (r)
    else:
        send_dump_cmd(IPADDR, PORT, filename)


if __name__ == "__main__":
    main()
