#ifndef _DEV_STRUCTS_H_
#define _DEV_STRUCTS_H_

#include <termios.h>
#include <sys/time.h>
#include <stdio.h>

#if !defined(_DEF_HRTIME_T_)
typedef long long hrtime_t;
#define _DEF_HRTIME_T_
#endif

#ifdef MMR
#include <linux/mcst/linux_mmr_io.h>
#endif

#ifdef MMRM
#include <linux/mcst/mmrm_io.h>
#endif

#ifdef MOKME
#include <mcst_rt.h>
#endif

#ifdef CAN
#include <net/if.h>
#endif

#ifdef cplusplus
extern "C" {
#endif

/*
 * Add defines of types (T_COM, T_MVP etc)
 */
#ifdef STP_DEFINED_TYPES
#include "../../include/dev_types.h"
#elif SYS_DEFINED_TYPES
#include <linux/mcst/dev_types.h>
#elif LOCAL_DEFINED_TYPES
#include "dev_types.h"
#endif

/*
 * Classes of devices
 */
#define C_SERIAL	0xdbf0001
#define C_ETH		0xdbf0002
#define C_CPU		0xdbf0003
#define C_MEMORY	0xdbf0005
#define C_VIDEO		0xdbf0006
#define C_BLOCK		0xdbf0007
#define C_FREQ		0xdbf0008
#define C_STREAM	0xdbf0009

/*
 * Other values for fields (look at comment's)
 */
#define R_PASS		0xfaa0001 // for result in mlistinfo_t
#define R_FAIL		0xfaa0002 // --

#define T_DEV_OPEN	0xfac0001 // for type in mlistinfo_t
#define T_DEV_INIT	0xfac0002 // --
#define T_DEV_WRITE	0xfac0003 // --
#define T_DEV_READ	0xfac0004 // --
#define T_DEV_IOCTL	0xfac0005 // --
#define T_DEV_POLL	0xfac0006 // --
#define T_DEV_PING	0xfac0007 // --
#define T_DEV_SELFTEST	0xfac0008 // --
#define T_DEV_CLOSE	0xfac0009 // --

/*
 * for signal devs
 */
#define ST_0		"0"
#define ST_1		"1"
#define NOT_STATE	"-1"

/*
 * for changes server/client
 */
#define SERVER_MODE	0xe340038
#define CLIENT_MODE	0xe340039
#define MAX_CNCTS	1

/*
 * for mokme and other device who return not (int) after open
 */
#define FAKE_DD		0xffff00

/*
 * for eth poll events:
 */
#define ETH_POLL_READ	0x12edf12
#define ETH_POLL_WRITE	0x12edf13

	typedef struct mcst_list_info {
		int time;
		int result;
		int type;
		struct mcst_device_info *next;
	} mlistinfo_t;

	typedef struct mcst_com_attributes {
		int		no_block_file_flag;
		int		read_timeout;
		int		write_timeout;
		int		poll_event;
		int		poll_revents;
		unsigned int	c_iflag;	/* input mode flags */
		unsigned int	c_oflag;	/* output mode flags */
		unsigned int	c_cflag;	/* control mode flags */
		unsigned int	c_lflag;	/* local mode flags */
		unsigned char	c_line;		/* line discipline */
		unsigned char	c_cc[NCCS];	/* control characters */
		unsigned int	c_ispeed;	/* input speed */
		unsigned int	c_ospeed;	/* output speed */
	} com_attr_t;

	typedef struct mcst_lpt_attributes {
 		int		read_timeout;
		int		write_timeout;
		int		no_block_file_flag;
	} lpt_attr_t;

	typedef struct mcst_eth_attributes {
		int		no_block_file_flag;
		int		mode;
		char		*ip_addr;
		int		port;
		int		type;
		int		family_addr;
		int		client;
		int		timeout;
		int		read_timeout;
		int		write_timeout;
		int		poll_event;
	} eth_attr_t;

#ifdef MVP
	typedef struct mcst_mvp_attributes {
		int		read_timeout;
		int		write_timeout;
		int		no_block_file_flag;
		int		poll_event;
		int		poll_revents;
	} mvp_attr_t;
#endif

#ifdef MPV
	typedef struct mcst_mpv_attributes {
		int		no_block_file_flag;
		int		poll_event;
		int		poll_revents;
		int		read_timeout;
		int		write_timeout;
	} mpv_attr_t;
#endif

#ifdef MOP
	typedef struct mcst_mop_attributes {
		int		no_block_file_flag;
		int		poll_event;
		int		poll_revents;
		int		read_timeout;
		int		write_timeout;
	} mop_attr_t;
#endif

#ifdef MBK3
	typedef struct mcst_mbk3_attributes {
		int		no_block_file_flag;
		char		*buffers;
		int		reset_timeout;
		int		write_timeout;
		int		read_timeout;
	} mbk3_attr_t;
#endif

#ifdef MOKM
	typedef struct mcst_mokm_attributes {
		int		no_block_file_flag;
		char		*read_buffers;
		char		*write_buffers;
		int		reset_timeout;
		int		write_timeout;
		int		read_timeout;
	} mokm_attr_t;
#endif

#ifdef MOKME
	typedef struct mcst_mokme_attributes {
		el_netdev_t	nd;
//		int		no_block_file_flag; //???
		int		client_terminate;
		int		read_proto;
		int		read_skipped;
		int		read_timeout;
		int		size; // for buffer
	} mokme_attr_t;
#endif

#ifdef MGPM
	typedef struct mcst_mgpm_attributes {
		int		write_timeout;
		int		read_timeout;
		int		no_block_file_flag;
	} mgpm_attr_t;
#endif

#if defined(MMR) || defined(MMRM)
#define MODE_CTRL	0x4e23311
#define MODE_TERM	0x4e23312
#define MODE_MNTR	0x4e23313
#endif

#ifdef MMR
	typedef struct mcst_mmr_attributes {
		int	no_block_file_flag;
		mmr_area_bozu_t *base_mmr;
		int	num_block_channel;
		int	block_error_device;
		int	block_read_command_term;
		int	time_interval_output_comm;
		int	solution_accept_cntrl;
		int	mode;
		int	init_state;
		int		write_timeout;
		int		read_timeout;

		mmr_iosubd_desc_t  *trans_buf_desc[MMR_BUF_USER_NUM];
		mmr_iosubd_desc_t  *reciv_buf_desc[MMR_BUF_USER_NUM];
		mmr_data_buf_t     *trans_data_buf[MMR_BUF_USER_NUM];
		mmr_data_buf_t     *reciv_data_buf[MMR_BUF_USER_NUM];
	} mmr_attr_t;
#endif

#ifdef MMRM
	typedef struct mcst_mmrm_attributes {
		int			no_block_file_flag;
		int			device_channel_number;
		mmrm_term_dev_adr_t	term_dev_adr;
		mmrm_term_dev_adr_t	term_subaddress;
		unsigned long*		base_mem;
		int			mode;
		int			poll_event;
		int			poll_revent;
		int			write_timeout;
		int			read_timeout;
	} mmrm_attr_t;
#endif

#ifdef CAN
	typedef struct mcst_can_attributes {
		unsigned long		write_address;
		unsigned long		read_address;
		unsigned long		stop_address;
		time_t			read_timeout;
		char			name[IFNAMSIZ];
	} can_attr_t;
#endif

#ifdef DSP

#define XYRAM_SIZE	( 0x1fff8 )
#define  PRAM_SIZE	( 0x7ff8 )

#define XYRAM_ADR	( 0x0 )
#define PRAM_ADR	( 0x40000 )


#define PADR		((instance * 0x400000) + PRAM_ADR)
#define XADR		((instance * 0x400000) + XYRAM_ADR)

#define DMA_DSP_MODE	0
#define DSP_TO_DSP_MODE	1

#define WR_TO_PRAM 	2
#define WR_TO_XYRAM	3

	typedef struct mcst_dsp_attributes {
		int	no_block_file_flag;
		int	poll_event;
		int	poll_revent;
		int	write_timeout;
		int	read_timeout;

		void	*xyram;	/* XYRAM current DSP-node */
		void	*pram;	/* PRAM current DSP-node */
		void	*dma;	/* DMA buffer */
		void	*batch;	/* for batch*/
		void	*l_reg;	/* local registers */
		void	*g_reg;	/* global registers */
		int	fd;	/* device handle */
		int	number;	/* device number in system */

		unsigned int words;
		unsigned int channel;
		unsigned long pram_adr;
		unsigned long xyram_adr;
		unsigned long src_adr;
		unsigned long dst_adr;
		unsigned long read_adr;

		int transferring_mode;
		int write_dest;
	} dsp_attr_t;
#endif

#ifdef RDMA
	typedef struct mcst_rdma_attributes {
		int	no_block_file_flag;
		int	poll_event;
		int	poll_revent;

		int	reqlen;
		int	acclen;
		char	*mmap_ptr;
		int	write_timeout;
		int	read_timeout;
	} rdma_attr_t;
#endif

/*
 * attributes for devices:
 */
	typedef union mcst_dev_attr {

		com_attr_t  com;
		lpt_attr_t  lpt;
		eth_attr_t  eth;
#ifdef MVP
		mvp_attr_t  mvp;
#endif
#ifdef MOP
		mop_attr_t  mop;
#endif
#ifdef MPV
		mpv_attr_t  mpv;
#endif
#ifdef MOKM
		mokm_attr_t mokm;
#endif
#ifdef MOKME
		mokme_attr_t mokme;
#endif
#ifdef MBK3
		mbk3_attr_t mbk3;
#endif
#ifdef MMR
		mmr_attr_t  mmr;
#endif
#ifdef MMRM
		mmrm_attr_t mmrm;
#endif
#ifdef DSP
		dsp_attr_t dsp;
#endif
#ifdef RDMA
		rdma_attr_t rdma;
#endif
#ifdef MGPM
		mgpm_attr_t mgpm;
#endif
#ifdef CAN
		can_attr_t can;
#endif
#ifdef GPS
		gps_attr_t gps;
#endif
	} mattr_t;

	typedef struct work_count {
		int pass;
		int fail;
		int used;
	} wcount_t;

	typedef struct mcst_device_info {
		wcount_t write;
		wcount_t read;
		wcount_t ioctl;
		wcount_t poll;
		wcount_t ping;
		wcount_t selftest;
	} minfo_t;

	typedef struct mcst_device_self {
		int info;
	} mcst_selftest_t;

	typedef struct mcst_device {
		int dd;
		int dev_class;
		int type;
		char path[1024];

		int (*mdev_init)(struct mcst_device*, mattr_t*);
		int (*mdev_open)(struct mcst_device*, int, int);
		int (*mdev_attr_set)(struct mcst_device*, mattr_t*);
		int (*mdev_attr_get)(struct mcst_device*, mattr_t*);
		int (*mdev_write)(struct mcst_device*, char*, int);
		int (*mdev_read)(struct mcst_device*, char*, int);
		int (*mdev_write_stop)(struct mcst_device*, char*, int);
		int (*mdev_check_stop)(struct mcst_device*, char*, int);
		int (*mdev_poll)(struct mcst_device**, int, int);
		int (*mdev_ioctl)(struct mcst_device*, int, void*);
		int (*mdev_ping)(struct mcst_device*);
		int (*mdev_get_info)(struct mcst_device*, minfo_t*);
		int (*mdev_close)(struct mcst_device*);

		mattr_t attr;
		minfo_t *info;
	} mdevice_t;

#define devclose(__fd)							\
	do {								\
		int __r = 0;						\
		do {							\
			__r = close(__fd);				\
			if (errno == EBADF || errno == EIO) break;	\
		} while (__r);						\
	} while (0)

#ifdef cplusplus
}
#endif

#endif
