#ifndef _LIBMCST_TEST_H_
#define _LIBMCST_TEST_H_

#ifdef cplusplus
extern "C" {
#endif

#ifndef _GNU_SOURCE
#define _GNU_SOURCE
#endif

#include <libintl.h>
#include <locale.h>
#include <time.h>
#include <sys/time.h>
#include <sched.h>
#include <semaphore.h>
#include <time.h>

#include "alginfo.h"
#include "dev_structs.h"
#include "dev_types.h"
#include "libmcst_dev.h"

#define _(s) gettext(s)

#define DIREC 2

#define SIZE32	32
#define SIZE64	64
#define SIZE128 128
#define SIZE256 256
#define SIZE512 512
#define KILOSIZE 1024
#define PAGESIZE 4096

#define PREF_POST_BUF 32

#define T_ALL 0xa11
#define PATH_USR "PATH_USR"
#define PATH_COMMON "PATH_COMMON"

/** test headers */
typedef struct stp_file_descr {
	int log;
	int res;
} stp_file_descr_t;

typedef struct stp_standard_settings {
	const char *type;
	const char *alg;
	unsigned long settings;
} stp_stand_settings_t;

typedef struct user_type {
	const char *type;
} stp_user_dev_type_t;

typedef struct initial_dev_type {
	char type[32];
	int spec;
} stp_init_devtype_t;

typedef struct device_types {
	const char *type;
	int num;
} stp_devtypes_t;

typedef struct all_types {
	const char *b_type;
	int num;
	const char *l_type;
	const char *description;
	int spec;
} stp_all_types_t;

typedef struct path_to_lib {
	char dir[KILOSIZE];
	char to_lib[2][KILOSIZE];
} stp_path_to_lib_t;


typedef struct time_result {
	float 	all;		// time of working function(write, read, poll...)
	float 	sec;		// work time in seconds
	float 	nsec;		// work time in nanoseconds
	struct timespec begin, end;
} stp_time_result_t;

typedef struct test_action {
	float 	current;	/* last transfer bytes */
	float 	bytes;		/* all bytes summary */
	float 	size;		/* size in specify value (Mb, etc) */
	int 	pos;		/* value position for size */
	float 	speed;		/* transfer speed in specify value (Mb, etc) */
	int 	speed_pos;	/* value position for speed */
	stp_time_result_t time;
} stp_action_t;

/** structure for measure speed transmitted messages */
typedef struct test_result {
	int	last;		// last result last action, temporary
	char 	s_value[6][20];	// string to specify speed (Mb/sec, etc)
	char    hz_str[6][10];	// string to specify frequency (Hz, KHz, ...)
	stp_action_t write;
	stp_action_t read;
} stp_result_t;


/** structure for output */
typedef struct info_print {
	char prefix[PREF_POST_BUF];
	char postfix[PREF_POST_BUF];
	char b_prefix[PREF_POST_BUF];
	char b_postfix[PREF_POST_BUF];
	char hostname[PREF_POST_BUF];
	int res_file;
} stp_print_t;

/** structure for get current time and data */
typedef struct current_time {
	time_t year;
	time_t month;
	time_t day;
	time_t hh;
	time_t mm;
	time_t ss;
} stp_time_t;

/** structure for work with functions to print errors and warnings */
typedef struct info_flag {
	char debug;
	char note;
	char warning;
	char error;
} stp_info_t;

typedef struct stp_lib {
	char *name;
	struct stp_lib *next;
} stp_lib_t;

typedef struct test_time {
	time_t start;
	time_t current;		// test's start and end time
} stp_test_time_t;

typedef struct test_passes {
	int n;			// ammount of passes int test
	int n_write;		// ammount of writings in duplex_mode
	int n_read;		// ammount of readings in duplex mode
} stp_test_passes_t;

typedef struct stp_device stp_dev_t;
	
typedef struct test_thread {
	int flag; // ready to work
	pthread_t tid; // thread id
	clockid_t clock_id; // CLOCK_MONOTONIC or CLOCK_REALTIME
  	pthread_condattr_t cattr; // attribute for setup CLOCK_MONOTONIC
	pthread_cond_t start; // cond for start work
	pthread_cond_t stop; // cond for end work
	sem_t sem;
	struct timespec timeout;
	void *(*thrd)(void *);
	int (*work)(stp_dev_t *);
	char name[32];
} stp_test_thread_t;
	
  typedef struct test_control {
	stp_test_thread_t write;
	stp_test_thread_t read;
	stp_test_thread_t poll;
} stp_test_control_t;

/** basic structure for attributes from user and different flags for test */
typedef struct stp_attr {
	int pid;
	int time_info;
	char *cmd;		// path to file with commands
	char *path_dev;		// path to device
	char *path_file;	// path to file (for external data)
	int rts;
	int port;		// port
	int time;
	int duration;
	int number;		// amount of test's iterations
	int test_affinity;	// number of proseccor, which will be run in test
	int test_errors;	// ammount of specified errors
	int test_delay;		// delay between test iteration
	int second_delay;	// delay between exchanges in seconds
	int milli_delay;	// delay between exchanges in milliseconds
  	int micro_delay;	// delay between exchanges in microseconds
	int timeout;		// Timeout for all system functions
	int write_timeout;	// Timeout for write
	int read_timeout;	// Timeout for read
	int poll_timeout;	// Timeout for poll
	int ioctl_timeout;	// Timeout for ioctl
	int poll;
	int dev_type;		// type of device
	int spec;		// specialized device or not
	int connect_timeout;	// Timeout for connecting to server from client
	int realtime_flag;	// fag to run test in RT mode
	int server_flag;	// Server test
	int client_flag;	// Client test
	int silent_flag;	// flag to run without output information
	int logoff;		// if unset - save log into file
  	int resoff;		// if unset - save res into file
	int verbose_flag;	// flag to run wiht debug information
	int debug_flag;		// flag to run test in debug mode
	int nrm_flag; 		// flag to run test in normal(usual) mode
	int speed;		// speed of exchanges between devices (for serial test)
	int udp;		// udp flag for eth test
	/* set full mode for test
	 * example: videotest - fullscreen */
	int full;
  	int force;		// set force flag (for HDD and may be other)
	int unforce;		// set unforce flag (for clear FORCE flag)
	int screen;		// set screen number for output data (for touchscreen)
	int algorithm;		// int algorithm for test
	int *write_array;	// message for write
	int *read_array;	// message for read
	char *poll_event;	// flag to install of poll (server or client)
	int wrcount;		// count for writings in duplex mode helps
				//   to check number of message
	int rdcount;		// count for readings in duplex mode helps
				//   to check number of message
	int size_of_buf;	// size of message for sending to device
	int index_msg;		// index for message's type
	int skip_compare;	// skip compare array's
	int size_msg;		// internal value for store array size at sizeof(int)
	int single_read;	// internal value. set for one read from any device
	sem_t sem;		// semaphore for synchronization in threads in duplex mode
	int silent_res;
	int print_info;		// flag to show all algorithms, which can be runned with this type
	int print_types; 	// flag to show all types of devices, which can be runned in test
	int rus;
	int eng;
	int empty_flag;
	int user_print;
	int alone;
	char *display;
	char *ext_path;
	void *library_handler;
	char full_path[KILOSIZE];
	char lib_path[KILOSIZE];
	char dir_name[KILOSIZE];	// name of directory with logfiles
	char path_to_logfile[KILOSIZE]; // path to logfile with test's results
	char alg[SIZE256];		// char algorithm for test
	char type[SIZE64];		// type of device
	char ip[SIZE32];		// ip adress
} stp_attr_t;

typedef union stp_userset {
	unsigned long r;
} stp_userset_t;

typedef struct test_action_res {
	int	only; 		// ammount of writings to device
	int	data;		// ammount of correct writings
} stp_test_action_t;

typedef struct test_error_res {
	int	all;	// amount of test's errors
	int 	cor_read;// ammount of not correct read from device
	int 	write;	// ammount of test write's errors
	int 	read;	// ammount of test read's errors
	int 	poll;	// ammount of test poll's errors
	int 	ioctl;	// ammount of test ioctl's errors
	int 	system;	// ammount of test system's errors
} stp_test_error_t;

/** structure for printing test's results */
typedef struct stp_res {
	int	time;
	int 	warning;	// ammount of test warning's
	int 	success;
	int 	skip;		// test finish with strange result, not error
	int	stop;		// for server/client mode - stop flag
	stp_test_action_t write;
	stp_test_action_t read;
	stp_test_error_t  error;
} stp_res_t;

/** basic structure for work with device, contains other structures for easy way to use test */
//typedef
struct stp_device {
	int		dd;		// device descriptor
	int		sigpipe;
	int		sigterminate;
	int		memory_allocation_error;
	unsigned long	*devmask;
	stp_test_passes_t pass;
	stp_lib_t	*usr;
	stp_lib_t	*common;
	stp_lib_t	*algors;
	stp_attr_t	attr;
	stp_res_t	res;
	stp_result_t	result;		/* store exchange results */
	stp_time_t	time;
	stp_print_t	print;
	stp_info_t	info;
	stp_path_to_lib_t path;
	stp_test_time_t	a_time;
	stp_test_time_t	t_time;
	stp_test_control_t work;
	stp_init_devtype_t init;
	stp_all_types_t	*all;
	stp_devtypes_t	*spec;
	stp_devtypes_t	*non_spec;
	stp_devtypes_t	*pl_event;
	stp_devtypes_t	*algs;
	stp_userset_t	input;
	stp_file_descr_t fd;
	stp_user_dev_type_t *user;
	void		*so_handler;
};// stp_dev_t;

//void full_string_for_output(char *string[], char *types);
	void init_language(char *path_to_main, char *package, char *value);
	void set_language(int lang, char *value);
	void print_timeout_info(stp_dev_t *dev, void (*fp)(stp_dev_t *dev));
	int set_attr(stp_dev_t *dev);
	int set_realtime(stp_dev_t *dev);
	void check_error (stp_dev_t *dev);
#ifdef UTEST_POLL_ON
	void test_poll (stp_dev_t *dev);
#endif
	void micro_sleep(stp_dev_t *dev);
	int scheduler(stp_dev_t *dev);
	int set_affinity(stp_dev_t *dev);
	int check(stp_dev_t *dev);
	int compare_array (int *rcv, int *xmit, int size, stp_dev_t *dev);
	void check_error_system(stp_dev_t *dev);
	void stp_print_result(stp_dev_t *dev);
	void set_type_all(stp_dev_t *dev);
	int complete_message(stp_dev_t *dev);
	void start_alg_time(stp_dev_t *dev);
	void end_alg_time(stp_dev_t *dev);
	int algorithm_run_disable(stp_dev_t *dev, char *user[]);
	int check_dev_bitmask(stp_dev_t *dev);
	void start_alg_time(stp_dev_t *dev);
	void current_alg_time(stp_dev_t *dev);
	int info_algs_types(stp_dev_t *dev, char *path_to_dir);
	void info_libmcst_test();
	void shortinfo_libmcst_test();

	char *get_device_type(stp_dev_t *dev);
	int device_opened(stp_dev_t *dev);

	void set_sigpipe_for_device(stp_dev_t *dev);
	int  get_sigpipe_from_device(stp_dev_t *dev);
	void clear_sigpipe_for_device(stp_dev_t *dev);

	void set_sigterminate(stp_dev_t *dev);
	int  get_sigterminate(stp_dev_t *dev);
	void clear_sigterminate(stp_dev_t *dev);

#ifdef cplusplus
}
#endif

#endif
