# -*- coding: utf-8 -*-

from PyQt4 import QtGui, QtCore

import res_settings
import date_panel
import vk_selection
import test_selection


class ControlPanel(QtGui.QWidget):
    update_text = QtCore.pyqtSignal(name='UpdateText')

    def __init__(self, settings):
        super(ControlPanel, self).__init__()
        self.settings = settings

        all_time_flag = QtGui.QCheckBox('All time')
        all_time_flag.setChecked(True)
        all_time_flag.stateChanged.connect(self.all_time_handler)

        self.show_mode_names = res_settings.show_mode_names
        show_mode_group = QtGui.QGroupBox('show mode')
        show_mode_group.setAlignment(QtCore.Qt.AlignHCenter)
        # for generate signal from radiobuttons
        self.show_mode_buttons = QtGui.QButtonGroup()
        self.show_mode_buttons.buttonClicked.connect(self.show_mode_handler)

        show_mode_layout = QtGui.QHBoxLayout()
        show_mode_layout.setContentsMargins(0, 0, 0, 0)

        for name in self.show_mode_names:
            item = QtGui.QRadioButton(name)
            if name == self.show_mode_names[0]:
                item.setChecked(True)
            show_mode_layout.addWidget(item)
            self.show_mode_buttons.addButton(item)
        show_mode_group.setLayout(show_mode_layout)

        self.data_start = date_panel.DatePanel('start')
        self.data_start.setSizePolicy(QtGui.QSizePolicy.Fixed,
                                 QtGui.QSizePolicy.Minimum)
        self.data_start.setEnabled(not all_time_flag.checkState())
        self.data_start.date_chaged.connect(self.date_start_handler)

        self.data_end = date_panel.DatePanel('end')
        self.data_end.setSizePolicy(QtGui.QSizePolicy.Fixed,
                                 QtGui.QSizePolicy.Minimum)
        self.data_end.setEnabled(not all_time_flag.checkState())
        self.data_end.date_chaged.connect(self.date_end_handler)

        data_box = QtGui.QVBoxLayout()
        data_box.addWidget(self.data_start)
        data_box.addWidget(self.data_end)
        data_box.setContentsMargins(0, 0, 0, 0)

        top_box = QtGui.QHBoxLayout()
        top_box.addWidget(all_time_flag)
        top_box.addLayout(data_box)
        top_box.setContentsMargins(0, 0, 0, 0)

        self.vk_sel = vk_selection.VkSelection(self.settings)
        self.vk_sel.setSizePolicy(QtGui.QSizePolicy.Expanding,
                                  QtGui.QSizePolicy.Fixed)
        self.vk_sel.stateChanged.connect(self.vk_selection_state_handler)
        # default
        self.vk_sel.setEnabled(False)

        self.test_sel = test_selection.TestSelection(self.settings)
        self.test_sel.setSizePolicy(QtGui.QSizePolicy.Expanding,
                                    QtGui.QSizePolicy.Fixed)
        self.test_sel.stateChanged.connect(self.test_selection_state_handler)
        # default
        self.test_sel.setEnabled(False)

        main_box = QtGui.QVBoxLayout()
        main_box.addWidget(show_mode_group)
        main_box.addLayout(top_box)
        main_box.addWidget(self.vk_sel)
        main_box.addWidget(self.test_sel)
        main_box.setContentsMargins(0, 0, 0, 0)
        main_box.addStretch()

        self.setLayout(main_box)

    def all_time_handler(self, item):
        if item:
            self.data_start.setEnabled(False)
            self.data_end.setEnabled(False)
            self.settings.set_all_time()
        else:
            self.data_start.setEnabled(True)
            self.data_end.setEnabled(True)
            self.settings.set_data_start(*self.data_start.get_date())
            self.settings.set_data_end(*self.data_end.get_date())
        self.update_vk_and_test_selections()
        self.update_text.emit()

    def show_mode_handler(self, item):
        name = str(item.text())
        self.settings.set_show_mode(name)
        if name == self.show_mode_names[0]:
            self.vk_sel.setEnabled(False)
            self.test_sel.setEnabled(False)
        elif name == self.show_mode_names[1]:
            self.vk_sel.setEnabled(True)
            self.test_sel.setEnabled(False)
        elif name == self.show_mode_names[2]:
            self.vk_sel.setEnabled(True)
            self.test_sel.setEnabled(True)
        self.update_text.emit()

    def date_start_handler(self, day, month, year):
        self.settings.set_data_start(day, month, year)
        self.update_vk_and_test_selections()
        self.update_text.emit()

    def date_end_handler(self, day, month, year):
        self.settings.set_data_end(day, month, year)
        self.update_vk_and_test_selections()
        self.update_text.emit()

    def vk_selection_state_handler(self):
        self.update_text.emit()

    def test_selection_state_handler(self):
        self.update_text.emit()

    def update_vk_and_test_selections(self):
        self.vk_sel.update_list_items()
        self.test_sel.update_list_items()


__author__ = 'alexey'
