# -*- coding: utf-8 -*-

import calendar
import datetime
from PyQt4 import QtGui, QtCore


class DatePanel(QtGui.QWidget):
    date_chaged = QtCore.pyqtSignal(int, int, int, name='DateChanged')

    def __init__(self, title=None):
        super(DatePanel, self).__init__()
        date = datetime.datetime.now()

        group_box = QtGui.QGroupBox(title)

        self.year = QtGui.QSpinBox(self)
        self.year.setMaximum(date.year)
        self.year.setMinimum(datetime.MINYEAR)
        self.year.setValue(date.year)
        self.year.valueChanged.connect(self.year_handler)

        self.month = QtGui.QComboBox(self)
        monthname = [QtCore.QString(unicode(calendar.month_name[m], 'utf-8'))
                     for m in range(1, 13)]
        self.month.addItems(monthname)
        self.month.setCurrentIndex(date.month - 1)
        self.month.currentIndexChanged.connect(self.month_handler)

        self.day = QtGui.QSpinBox(self)
        self.day.setMinimum(1)
        self.day.setMaximum(calendar.monthrange(date.year, date.month)[1])
        self.day.setValue(date.day)
        self.day.valueChanged.connect(self.year_handler)

        date_box = QtGui.QHBoxLayout()
        date_box.addWidget(self.day)
        date_box.addWidget(self.month)
        date_box.addWidget(self.year)
        date_box.setContentsMargins(0, 0, 0, 0)

        group_box.setLayout(date_box)

        box = QtGui.QVBoxLayout()
        box.addWidget(group_box)
        box.setContentsMargins(0, 0, 0, 0)
        self.setLayout(box)

    def get_date(self):
        return (self.day.value(), self.month.currentIndex() + 1,
                self.year.value())

    def date_correct(self):
        max_day = calendar.monthrange(self.year.value(),
                                      self.month.currentIndex() + 1)[1]
        if max_day < self.day.value():
            self.day.setValue(max_day)

    def year_handler(self, year):
        self.date_correct()
        self.date_chaged.emit(self.day.value(), self.month.currentIndex() + 1,
                              self.year.value())

    def month_handler(self, month):
        self.date_correct()
        self.date_chaged.emit(self.day.value(), self.month.currentIndex() + 1,
                              self.year.value())

    def day_handler(self, day):
        self.date_correct()
        self.date_chaged.emit(self.day.value(), self.month.currentIndex() + 1,
                              self.year.value())

__author__ = 'alexey'
