# -*- coding: utf-8 -*-

from PyQt4 import QtGui, QtCore

import control_panel
import text_viewer
import res_settings
import script_worker


class CentralWidget(QtGui.QWidget):
    def __init__(self):
        super(CentralWidget, self).__init__()
        self.set = res_settings.Settings()

        self.panel = control_panel.ControlPanel(self.set)
        self.panel.setSizePolicy(QtGui.QSizePolicy.Fixed,
                                 self.panel.sizePolicy().verticalPolicy())
        self.panel.update_text.connect(self.update_text)

        self.text = text_viewer.TextViewer()
        self.update_text()

        box = QtGui.QHBoxLayout()
        box.addWidget(self.panel)
        box.addWidget(self.text)

        self.setLayout(box)

    def update_text(self):
        self.text.clear()
        try:
            lines = script_worker.get_results(self.set)
        except script_worker.ScritpWorkerExeption as err:
            self.text.text_bad_request()
            self.text.text_error(str(err))
        else:
            if len(lines):
                for l in lines:
                    self.text.append(l)
            else:
                self.text.text_bad_request()

    def set_path_to_results(self, path):
        self.set.set_path_to_res(path)

class MainWindow(QtGui.QMainWindow):
    def __init__(self):
        super(MainWindow, self).__init__()
        self.setWindowTitle('STP Result viewer')
        self.setMinimumSize(640, 480)
        resolution = QtGui.QDesktopWidget().screenGeometry()
        self.resize(resolution.width(), resolution.height())

        self.create_menu()

        central = CentralWidget()
        self.setCentralWidget(central)

    def create_menu(self):
        menu_bar = self.menuBar()

        exit_action = QtGui.QAction('&Exit', self)
        exit_action.triggered.connect(QtGui.qApp.quit)

        set_dir_action = QtGui.QAction('&Set path to results', self)
        set_dir_action.triggered.connect(self.show_dialog)

        about_action = QtGui.QAction('&About', self)
        about_action.triggered.connect(self.show_about)

        menu_file = menu_bar.addMenu('&File')
        menu_file.addAction(set_dir_action)
        # menu_file.addAction('&Open')
        menu_file.addAction(exit_action)

        menu_help = menu_bar.addMenu('&Help')
        menu_help.addAction(about_action)

    def show_about(self):
        body =  '''<p><center><h1>%s</h1></center></p>
                <center><font size = "4" >Program allows to make a request to view the test results</font>
                <p><font size = "3" >2013 - 2014 Alexey Safiullin</font></p></center>
                <center><font size = "3" ><a href ="http://www.mcst.ru/">http://www.mcst.ru/</a></font></center><body>''' % (str(self.windowTitle()))
        QtGui.QMessageBox.about(self, 'About', body)

    def show_dialog(self):
        dialog = QtGui.QFileDialog(self, caption='Results folder',
                                   directory=res_settings.DEFAULT_PATH_TO_RESULTS)
        dialog.setFileMode(QtGui.QFileDialog.Directory)
        main_rect = self.geometry()
        dialog.setGeometry(main_rect.x() + 10, main_rect.y() + 10,
                           main_rect.width() - 20, main_rect.height() - 20)
        if dialog.exec_():
            self.centralWidget().set_path_to_results(str(dialog.selectedFiles()[0]))
            self.centralWidget().update_text()
            self.centralWidget().panel.update_vk_and_test_selections()


__author__ = 'alexey'
