"""
Code related to clipboard
"""

from copy import deepcopy
from PyQt5.QtCore import Qt

class Clipboard:
    """
    This class implements a clipboard.
    """

    def __init__(self):
        self.data = []
        self.level = 0

    def copy(self, data, level):
        self.data = deepcopy(data)
        self.level = level

    def get_data(self, tree):
        """
        get data from clipboard
        @param tree: TreeWidget which wants data
        """
        if len(self.data) > 0 and self.level == 1:

            ret_data = deepcopy(self.data)
            root_node = tree.topLevelItem(0)
            existing_nodes = []
            for i in range(0, root_node.childCount()):
                node = root_node.child(i)
                path_str = str(node.data(0, Qt.UserRole))
                existing_nodes.append(path_str.split('/')[-1])
            for host in ret_data:
                if host.hostname in existing_nodes:
                    if host.hostname.split('.')[-1].isdigit():
                        # if we have number after dot in host name
                        num = int(host.hostname.split('.')[-1])
                        while host.hostname in existing_nodes:
                            num += 1
                            host.hostname = host.hostname.rsplit('.', 1)[0] + '.' + str(num)
                        existing_nodes.append(host.hostname)
                    else:
                        host.hostname = host.hostname + '.0'
                        num = 0
                        while host.hostname in existing_nodes:
                            num += 1
                            host.hostname = host.hostname.rsplit('.', 1)[0] + '.' + str(num)
                        existing_nodes.append(host.hostname)
            return ret_data, self.level
        elif len(self.data) > 0 and self.level == 2:

            ret_data = deepcopy(self.data)
            path = str(tree.currentItem().data(0, Qt.UserRole))
            hostname = path.split('/')[1]
            host_node = tree.getNodeByPath('/' + hostname)
            existing_nodes = []
            for i in range(0, host_node.childCount()):
                node = host_node.child(i)
                path_str = str(node.data(0, Qt.UserRole))
                existing_nodes.append(path_str.split('/')[-1])

            for test in ret_data:
                if test.testname in existing_nodes:
                    if test.testname.split('.')[-1].isdigit():
                        # if we have number after dot in test name
                        num = int(test.testname.split('.')[-1])
                        while test.testname in existing_nodes:
                            num += 1
                            test.testname = test.testname.rsplit('.', 1)[0] + '.' + str(num)
                        existing_nodes.append(test.testname)
                    else:
                        test.testname = test.testname + '.0'
                        num = 0
                        while test.testname in existing_nodes:
                            num += 1
                            test.testname = test.testname.rsplit('.', 1)[0] + '.' + str(num)
                        existing_nodes.append(test.testname)

            return ret_data, self.level
        else:
            return None, None
