#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
GTK GUI Viewer

 ----------------
| windowtitle [X]|
+----------------+
|       |        |
|  cfg  |  opt   |
|  view |  view  |
|       |        |
+----------------+
|    loglist     |
+----------------+
|   statusbar    |
 ----------------
"""


import datetime
import os
import sys
try:
    import pygtk
    pygtk.require("2.0")
except:
    pass
try:
    import gtk
    import gtk.glade
except:
    sys.exit(1)

from stp_gui.StpControl import StpControl
from Common import APP_NAME, APP_VERSION


#------------------------------------------------------------------------------
class windowtitle:
    """Set Window Title: [RUN] <*><file> - <path> - <APP_NAME>"""

    def __init__(self, widget):
        self.__window = widget
        self.__run = False
        self.__changed = False
        self.__fname = None

    def windowtitle_update_run(self, run=False):
        """
        Update 'Test Run' flag

        @param run: set Run flag
        @type run: bool

        @raise: no special exceptions

        @return: no return
        @rtupe: None
        """
        self.__run = run
        self.windowtitle_update()

    def windowtitle_update_changed(self, changed=True):
        """Update 'File Changed' flag"""
        self.__changed = changed
        self.windowtitle_update()

    def windowtitle_update_file(self, filename=None, changed=False):
        """Update Filename and 'File Changed' flag"""
        self.__fname = filename
        self.__changed = changed
        self.windowtitle_update()

    def windowtitle_update(self):
        """Update message in window title"""
        if not self.__fname:
            self.__window.set_title(APP_NAME)
        else:
            self.__window.set_title("{} {}{} - {} - {}". \
                format((lambda x: "RUN" if x else "")(self.__run), \
                       (lambda x: "*" if x else "")(self.__changed), \
                       os.path.basename(self.__fname), \
                       os.path.dirname(self.__fname), \
                       APP_NAME))


#------------------------------------------------------------------------------
class cfgview:
    """cfgview control"""

    # singleton
    def __new__(cls, *args, **kw):
        if not hasattr(cls, '_instance'):
            cls._instance = super(cfgview, cls).__new__(cls, *args, **kw)
        return cls._instance


    def __init__(self, widget):
        self.__treestore = gtk.TreeStore(str, str)
        self.__treeview = gtk.TreeView(model = self.__treestore)
        self.__treeview.set_grid_lines(gtk.TREE_VIEW_GRID_LINES_BOTH)
        # create columns
        # - 1
        nm_renderer = gtk.CellRendererText()
        #nm_renderer.set_property('editable', True)
        nm_column = gtk.TreeViewColumn("Name", nm_renderer, text=0)
        self.__treeview.append_column(nm_column)
        # - 2
        st_renderer = gtk.CellRendererText()
        st_column = gtk.TreeViewColumn("St", st_renderer, text=1)
        self.__treeview.append_column(st_column)
        # - 3
        tg_renderer = gtk.CellRendererToggle()
        tg_renderer.set_property('activatable', True)
        tg_column = gtk.TreeViewColumn("En", tg_renderer, text=2)
        self.__treeview.append_column(tg_column)
        # - 4
        all_renderer = gtk.CellRendererText()
        all_column = gtk.TreeViewColumn("All", all_renderer, text=3)
        self.__treeview.append_column(all_column)
        # - 5
        tm_renderer = gtk.CellRendererText()
        tm_column = gtk.TreeViewColumn("Time", tm_renderer, text=4)
        self.__treeview.append_column(tm_column)
        # - 6
        ok_renderer = gtk.CellRendererText()
        ok_column = gtk.TreeViewColumn("OK", ok_renderer, text=5)
        self.__treeview.append_column(ok_column)
        # - 7
        err_renderer = gtk.CellRendererText()
        err_column = gtk.TreeViewColumn("Err", err_renderer, text=6)
        self.__treeview.append_column(err_column)
        # - 8
        # TODO: progress
        p_renderer = gtk.CellRendererText()
        p_column = gtk.TreeViewColumn("Progress", p_renderer, text=7)
        self.__treeview.append_column(p_column)
        #
        #widget.add(self.__treeview)
        widget.add_with_viewport(self.__treeview)
        widget.show_all()

    def new_tree(self, cfg):
        # TODO: clead tree
        self.add_element(cfg, None, "hosts", 0)

    def add_element(self, cfg, parent, name, lvl):
        if name == "options":
            return
        me = self.__treestore.append(parent, [name, ''])
        if lvl > 1:
            return
        #for ch in cfg[name].keys():
        #    self.add_element(cfg[name], me, ch, lvl+1)


#------------------------------------------------------------------------------
class optview:
    """options control"""

    # singleton
    def __new__(cls, *args, **kw):
        if not hasattr(cls, '_instance'):
            cls._instance = super(optview, cls).__new__(cls, *args, **kw)
        return cls._instance


    def __init__(self, widget):
        self.__table = gtk.Table(2, 2, False)
        self.__frame = gtk.Frame("Options")

        #DEBUG:

        self.__table.attach(self.__frame, 0, 1, 0, 1)
        widget.add_with_viewport(self.__table)
        widget.show_all()


#------------------------------------------------------------------------------
class loglist:
    """Log List control"""

    def __init__(self, widget):
        self.__linecnt = 0
        self.__liststore = gtk.ListStore(gtk.gdk.Pixbuf, str, str)
        self.__treeview = gtk.TreeView(model = self.__liststore)
        self.__treeview.set_grid_lines(gtk.TREE_VIEW_GRID_LINES_BOTH)
        # icons
        # TODO: get icons
        self.__symbol = None #[]
        #self.__symbol.append(gtk.STOCK_OPEN) #gtk.IconTheme.load_icon("gtk-cut", 64, 0)
        # create columns
        # - 1
        self.__px_renderer = gtk.CellRendererPixbuf()
        px_column = gtk.TreeViewColumn("*", self.__px_renderer)
        self.__treeview.append_column(px_column)
        # - 2
        self.__tim_renderer = gtk.CellRendererText()
        tim_column = gtk.TreeViewColumn("Time", self.__tim_renderer, text=1)
        self.__treeview.append_column(tim_column)
        # - 3
        self.__str_renderer = gtk.CellRendererText()
        str_column = gtk.TreeViewColumn("Message", self.__str_renderer, text=2)
        self.__treeview.append_column(str_column)
        #
        widget.add_with_viewport(self.__treeview)
        widget.show_all()

    def loglist_addmsg(self, msg, symbol=0):
        """Add messege to log list"""
        ## TODO: linecolor
        #if self.__linecnt % 2:
        #    self.__px_renderer.set_property('cell-background', 'white')
        #    self.__tim_renderer.set_property('cell-background', 'white')
        #    self.__str_renderer.set_property('cell-background', 'white')
        #else:
        #    self.__px_renderer.set_property('cell-background', 'lightgray')
        #    self.__tim_renderer.set_property('cell-background', 'lightgray')
        #    self.__str_renderer.set_property('cell-background', 'lightgray')
        # TODO: autoscroll to end of list
        self.__liststore.append([self.__symbol,
                                datetime.datetime.now(), msg])
        #self.__liststore.insert(0, [self.__symbol,
        #                        datetime.datetime.now(), msg])
        self.__linecnt += 1


#------------------------------------------------------------------------------
class statusbar:
    """Statusbar control"""

    def __init__(self, widget):
        self.__sb = widget
        self.__context_id = self.__sb.get_context_id("id1")

    def statusbar_pushmsg(self, msg):
        """Add messege to statusbar lifo"""
        self.__sb.push(self.__context_id, msg)

    def statusbar_popmsg(self):
        """Remove messege from statusbar lifo"""
        self.__sb.pop(self.__context_id)

    def statusbar_replasemsg(self, msg):
        """Replace last message in statusbar"""
        self.__sb.pop(self.__context_id)
        self.__sb.push(self.__context_id, msg)


#------------------------------------------------------------------------------
class GUIGTK(windowtitle, loglist, statusbar):
    """This is an stp_gui application"""

    # singleton
    def __new__(cls, *args, **kw):
        if not hasattr(cls, '_instance'):
            cls._instance = super(GUIGTK, cls).__new__(cls, *args, **kw)
        return cls._instance


    def __init__(self):
        # GUI CONSTRUCTION CODE
        self.__builder = gtk.Builder()
        self.__gladefile = os.path.join(os.path.dirname(__file__), 
                                        "GUIGTK.glade")
        self.__builder.add_from_file(self.__gladefile)
        self.__builder.connect_signals(self)

        # Get GIU Objects
        self.__mainwindow = self.__builder.get_object("mainWindow")

        # Window Title, Log List, Statusbar
        windowtitle.__init__(self, self.__mainwindow)
        loglist.__init__(self, self.__builder.get_object("logScrolledWindow"))
        self.loglist_addmsg("GUI started")
        sbo = self.__builder.get_object("mainStatusbar")
        statusbar.__init__(self, sbo)

        # cfg tree
        cto = self.__builder.get_object("projScrolledWindow")
        self.__cfgview = cfgview(cto)

        # options
        #self.__optview = optview(self.__builder.get_object("optScrolledWindow"))

        # Test Controller
        self.__tstctrl = StpControl()
        self.new_cfg()


    ##------------------------------------
    ##          METHODS
    ##------------------------------------

    def chk_cfg(self):
        """Check save cfg and send question to user"""
        if self.__tstctrl.config_ischanged() == False:
            return False  # no changes
        md = gtk.MessageDialog(self.__mainwindow,
                               gtk.DIALOG_DESTROY_WITH_PARENT,
                               gtk.MESSAGE_QUESTION,
                               gtk.BUTTONS_YES_NO,
                               "Configuration changed, reject?")
        response = md.run()
        md.destroy()
        if response == gtk.RESPONSE_YES:
            return False  # ignore changes
        else:
            return True  # stay with this config

    def new_cfg(self):
        """Create empty cfg and clean interface"""
        if self.__tstctrl.config_create():
            self.__cfgview.new_tree(self.__tstctrl.get_config())
            self.windowtitle_update_file()
            self.loglist_addmsg("Set empty confuguration")
            self.statusbar_popmsg()

    def open_cfg(self, filename):
        """read cfg and update interface"""
        if self.__tstctrl.config_read(filename):
            self.__cfgview.new_tree(self.__tstctrl.get_config())
            self.windowtitle_update_file(filename)
            self.loglist_addmsg("Read configuration from file: %s"%(filename))
            self.statusbar_replasemsg(filename)
        else:
            md = gtk.MessageDialog(self.__mainwindow,
                                   gtk.DIALOG_DESTROY_WITH_PARENT,
                                   gtk.MESSAGE_WARNING,
                                   gtk.BUTTONS_CANCEL,
                                   "Can't open configuration file")
            md.run()
            md.destroy()

    def save_cfg(self, filename=None):
        """save cfg and update interface"""
        if self.__tstctrl.config_write(filename):
            fn = self.__tstctrl.config_getfilename()
            s = "Configuration saved to {}".format(fn)
            self.windowtitle_update_file(fn)
            self.loglist_addmsg(s)
            self.statusbar_replasemsg(s)
        else:
            md = gtk.MessageDialog(self.__mainwindow,
                                   gtk.DIALOG_DESTROY_WITH_PARENT,
                                   gtk.MESSAGE_WARNING,
                                   gtk.BUTTONS_CANCEL,
                                   "Can't write configuration file")
            md.run()
            md.destroy()

    def exit_dialog(self):
        """create exit dialog"""
        d = gtk.Dialog(title="Exit...",
                       parent=self.__mainwindow,
                       flags=gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
                       buttons=(gtk.STOCK_OK, gtk.RESPONSE_OK,
                                gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL))
        f = gtk.Frame()
        d.vbox.pack_start(f)
        # box for radio buttons
        box = gtk.VBox()
        f.add(box)
        first = None
        for text in ["1: Close GUI",
                     "2: Close GUI and Kernel",
                     "3: Close GUI, Kernel, Monitors"]:
            b = gtk.RadioButton(first, text)
            if not first:
                first = b
            box.pack_start(b, expand=False)
        d.set_default_response(gtk.RESPONSE_OK)
        d.show_all()
        result = d.run()
        # parse exit dialog result
        text = None
        if result == gtk.RESPONSE_OK:
            act = [radio for radio in first.get_group() if radio.get_active()]
            text = "%s" % act[0].get_label()
        d.destroy()
        # exit
        if text == None:
            return False  # disable exit
        self.do_exit(text[:1])
        return True  # exit

    def do_exit(self, mode):
        """close connections"""
        if mode == "3":
            self.__tstctrl.net_disconnect(kernclose=True, monclose=True)
        elif mode == "2":
            self.__tstctrl.net_disconnect(kernclose=True, monclose=False)
        else:
            self.__tstctrl.net_disconnect(kernclose=False, monclose=False)


    ##------------------------------------
    ##          MENU CALLBACK METHODS
    ##------------------------------------

    ## FILE MENU

    def on_actionFileNew(self, widget, data=None):
        """File - New: create empty configuation"""
        if self.chk_cfg():
            return None  # stay with this config
        self.new_cfg()

    def on_actionFileOpen(self, widget, data=None):
        """File - Open: read configuation from file"""
        if self.chk_cfg():
            return None  # stay with this config
        dialog = gtk.FileChooserDialog(title="Open",
                                       action=gtk.FILE_CHOOSER_ACTION_OPEN,
                                       buttons=(gtk.STOCK_CANCEL,
                                                gtk.RESPONSE_CANCEL,
                                                gtk.STOCK_OPEN,
                                                gtk.RESPONSE_OK))
        filter = gtk.FileFilter()
        filter.set_name("Cfg files")
        filter.add_pattern("*.cfg")
        dialog.add_filter(filter)
        response = dialog.run()
        if response == gtk.RESPONSE_OK:
            self.open_cfg(os.path.abspath(dialog.get_filename()))
        dialog.destroy()

    def on_actionFileSave(self, widget, data=None):
        """File - Save: save configuation in current file"""
        if not self.__tstctrl.config_getfilename():
            self.on_actionFileSaveAs(self, widget)
        else:
            self.save_cfg()

    def on_actionFileSaveAs(self, widget, data=None):
        """File - Save As: save configuation in new file"""
        dialog = gtk.FileChooserDialog(title="Save As..",
                                       action=gtk.FILE_CHOOSER_ACTION_SAVE,
                                       buttons=(gtk.STOCK_CANCEL,
                                                gtk.RESPONSE_CANCEL,
                                                gtk.STOCK_OPEN,
                                                gtk.RESPONSE_OK))
        if self.__tstctrl.config_getfilename():
            dialog.set_filename(self.__tstctrl.config_getfilename()[0])
        filter = gtk.FileFilter()
        filter.set_name("Cfg files")
        filter.add_pattern("*.cfg")
        dialog.add_filter(filter)
        response = dialog.run()
        if response == gtk.RESPONSE_OK:
            self.save_cfg(dialog.get_filename()[0])
        dialog.destroy()

    def on_actionFileClose(self, widget, data=None):
        """File - Close: identical File - New"""
        if self.chk_cfg():
            return None  # stay with this config
        self.new_cfg()

    def on_actionFileExit(self, widget, data=None):
        """File - Quit: check save cfg and exit"""
        if self.chk_cfg() == False:
            if self.exit_dialog():
                self.__mainwindow.destroy()

    ## TST MENU

    def on_actionTestConnect(self, widget, data=None):
        """request IP-address and connect"""
        d = gtk.Dialog(title="Connect...",
                       parent=self.__mainwindow,
                       flags = gtk.DIALOG_MODAL |
                               gtk.DIALOG_DESTROY_WITH_PARENT,
                       buttons=(gtk.STOCK_OK, gtk.RESPONSE_OK,
                                gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL))
        box = gtk.HBox(False, 0)
        d.vbox.pack_start(box)
        # Text label
        lb = gtk.Label("Enter IP-address: ")
        box.pack_start(lb, False, False, 0)
        # Set default text in text entry box
        #TODO: get current IP from StpControl
        entry_checker_default = "localhost"
        # Generate text entry box
        entry = gtk.Entry()
        entry.set_text(entry_checker_default)
        entry.select_region(0, len(entry.get_text()))
        box.pack_start(entry, False, False, 0)
        entry.connect("activate", lambda w: d.response(gtk.RESPONSE_OK))
        # dialog
        d.set_default_response(gtk.RESPONSE_OK)
        d.show_all()
        result = d.run()
        text = entry.get_text()
        d.destroy()
        # parse exit dialog result
        if result == gtk.RESPONSE_OK:
            self.__tstctrl.net_connect(text)

    def on_actionTestRunPause_toggled(self, widget, data=None):
        if widget.get_active():
            print("on_actionTestRunPause_toggled - pressed")
        else:
            print("on_actionTestRunPause_toggled - released")

    def on_actionTestStop(self, widget, data=None):
        print("on_actionTestStop pressed")

    def on_actionTestExclusive(self, widget, data=None):
        print("on_actionTestExclusive pressed")

    def on_actionTestReconnect(self, widget, data=None):
        print("on_actionTestReconnect pressed")

    ### def on_actionTestClosekernel
    ### - Завершить только Ядро

    ## VIEW MENU

    def on_actionViewMessages(self, widget, data=None):
        print("on_actionViewMessages pressed")

    def on_actionViewPreferences(self, widget, data=None):
        print("on_actionViewPreferences pressed")

    ## HELP MENU

    def on_actionHelpAbout(self, widget, data=None):
        """Help - About: create About dialod"""
        about = gtk.AboutDialog()
        about.set_program_name(APP_NAME)
        about.set_version(APP_VERSION)
        about.set_copyright("Copyright (c) 2015 MCST")
        about.set_comments("""
____###_#####_####____
___#______#___#___#___
____###___#___####____
_______#__#___#_______
___####___#___#_______
""")
        about.set_website("http://www.mcst.ru")
        about.set_authors(["Andrey.V.Kalita@mcst.ru", "и др."])
        try:
            logofile = os.path.join("{}/../pixmaps/logo.png". \
                format(os.path.dirname(__file__)))
            about.set_logo(gtk.gdk.pixbuf_new_from_file(logofile))
        finally:
            about.run()
            about.destroy()


    ##------------------------------------
    ##          WINDOW CALLBACK METHODS
    ##------------------------------------

    def on_mainWindow_delete(self, widget, event):
        """check save cfg and return True to disable exit"""
        if self.chk_cfg() == False:
            if self.exit_dialog():
                return False  # quit
            else:
                return True  # disable quit
        return True  # disable quit

    def on_mainWindow_destroy(self, widget):
        """destroy main window"""
        gtk.main_quit()


    ##------------------------------------
    def main(self):
        self.__mainwindow.show_all()
        gtk.main()
        sys.exit(0)


#------------------------------------------------------------------------------
if __name__ == "__main__":
    gui_instance = GUIGTK()
    gui_instance.main()

