#!/usr/bin/env python
# -*- coding: utf-8 -*-

class State:

    def __init__(self, bits, delimFull=None, delimShort=None, delimText=None):
        self.bits = bits

        if delimFull:
            self.delimFull = delimFull
        else:
            self.delimFull = '; '

        if delimShort:
            self.delimShort = delimShort
        else:
            self.delimShort = ' '

        if delimText:
            self.delimText = delimText
        else:
            self.delimText = ', '

        self.state = {
            0x0001: ('[R]', 'RUN'),
            0x0002: ('[P]', 'PAUSE'),
            0x0004: ('[E]', 'ERROR'),
            0x0008: ('[C]', 'CONTINUE'),
            0x0010: ('[S]', 'STOP'),
            0x0020: ('{C}', 'CONNECT'),
            0x0040: ('{T}', 'TERMINATE'),
            0x0080: ('{U}', 'UNAVAILABLE'),
            0x0100: ('{E}', 'CONNECT ERROR'),
            0x0200: ('{D}', 'DISCONNECT'),
            0x1000: ('{B}', 'BUSY'),
            0x400000: ('{V}', 'OTHER VERSION')
        }

    def getShortStatus(self, d=None, bits=None):
        s = ''
        if not d:
            d = self.delimShort
        if not bits:
            bits = self.bits
        for i in self.state:
            if bits & i:
                if not s:
                    s += self.state[i][0]
                    continue
                s += d + self.state[i][0]
        return s

    def getTextStatus(self, d=None, bits=None):
        s = ''
        if not d:
            d = self.delimText
        if not bits:
            bits = self.bits
        for i in self.state:
            if bits & i:
                if not s:
                    s += self.state[i][1]
                    continue
                s += d + self.state[i][1]
        return s

    def getFullStatus(self, d=None, bits=None):
        s = ''
        if not d:
            d = self.delimFull
        if not bits:
            bits = self.bits
        for i in self.state:
            if bits & i:
                if not s:
                    s += self.state[i][0] + ' - ' + self.state[i][1]
                    continue
                s += d + self.state[i][0] + ' - ' + self.state[i][1]
        return s

def main():
    print('262192 ---------------')
    s = State(262192, '\n')
    print(' 1 ' + s.getShortStatus())
    print(' 2 ' + s.getTextStatus())
    print(' 3 ' + s.getFullStatus())

    print('0 ---------------')
    s = State(0, ' ')
    print(' 1 ' + s.getShortStatus())
    print(' 2 ' + s.getTextStatus())
    print(' 3 ' + s.getFullStatus())

    print('-1 ---------------')
    s = State(-1, '\n', '\n')
    print(' 1 ' + s.getShortStatus())
    print(' 2 ' + s.getTextStatus())
    print(' 3 ' + s.getFullStatus())

if __name__ == "__main__":
    main()
