#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
import json

from stp_gui.Config import host, test
from stp_gui.StpNetwork import NetCmd

class StpControlMDF:
    def __init__(self, control):
        self._control = self
        super().__init__()

    def mdf_addvk(self, name, dump=False):
        """Добавить новый ВК в текущую конфигурацию"""
        logging.debug("Add VK: %s" % name)
        # MDF_ADD_VK

        if dump:
            h = host(json.loads(dump))
            message = '{"vkname":"' + name + '","vk":[' + h.dump(compact=True) + ']}'
            if self._control._connected:
                self._control._client.net_request(NetCmd.MDF_ADD_VK, message)
            else:
                self._control.diffConfig.emit(NetCmd.MDF_ADD_VK, message)
        else:
            h = host()
            h.hostname = name
            message = '{"vkname":"' + name + '","vk":[' + h.dump(compact=True) + ']}'
            if self._control._connected:
                self._control._client.net_request(NetCmd.MDF_ADD_VK, message)
            else:
                self._control.diffConfig.emit(NetCmd.MDF_ADD_VK, message)
        return True

    def mdf_chnvk(self, hostname, dump):
        """Изменить параметры указанного ВК"""
        """in dump parameter - dump of Config.host_options class"""
        logging.debug("Change VK: %s" % hostname)
        # MDF_CHN_VK

        h = host(json.loads(dump))
        message = '{"vkname":"' + hostname + '","vk":[' + h.dump(compact=True) + ']}'
        if self._control._connected:
            self._control._client.net_request(NetCmd.MDF_CHN_VK, message)
        else:
            self._control.diffConfig.emit(NetCmd.MDF_CHN_VK, message)

        return True

    def mdf_delvk(self, name):
        """Удалить указанный ВК из конфигурации"""
        logging.debug("Delete VK: %s" % name)
        # MDF_DEL_VK

        message = '{"vkname":"' + name + '"}'

        if self._control._connected:
            self._control._client.net_request(NetCmd.MDF_DEL_VK, message)
        else:
            self._control.diffConfig.emit(NetCmd.MDF_DEL_VK, message)

        return True

    def mdf_addtest(self, hostname, testname, dump=False):
        """Добавить новый тест в текущую конфигурацию"""
        logging.debug("Add test: %s in %s" % (testname, hostname))
        # MDF_ADD_TEST

        t = None
        if dump:
            t = test(json.loads(dump))
            t.testname = str(testname)
        else:
            t = test()
            t.testname = str(testname)

        message = '{"vkname":"' + hostname + '",' + \
                  '"testname":"' + testname + '",' \
                                              '"test":[' + t.dump(compact=True) + ']}'

        if self._control._connected:
            self._control._client.net_request(NetCmd.MDF_ADD_TEST, message)
        else:
            self._control.diffConfig.emit(NetCmd.MDF_ADD_TEST, message)

        return True

    def mdf_chntest(self, hostname, testname, dump):
        """Изменить параметры указанного теста"""
        logging.debug("Change test: %s in %s" % (testname, hostname))
        # MDF_CHN_TEST

        t = test(json.loads(dump))

        message = '{"vkname":"' + hostname + '",' + \
                  '"testname":"' + testname + '",' \
                                              '"test":[' + t.dump(compact=True) + ']}'

        if self._control._connected:
            self._control._client.net_request(NetCmd.MDF_CHN_TEST, message)
        else:
            self._control.diffConfig.emit(NetCmd.MDF_CHN_TEST, message)

        return True

    def mdf_deltest(self, hostname, testname):
        """Удалить указанный тест из конфигурации"""
        logging.debug("Delete test: %s in %s" % (testname, hostname))
        # MDF_DEL_TEST

        message = '{"vkname":"' + hostname + '",' + \
                  '"testname":"' + testname + '"' + \
                  '}'

        if self._control._connected:
            self._control._client.net_request(NetCmd.MDF_DEL_TEST, message)
        else:
            self._control.diffConfig.emit(NetCmd.MDF_DEL_TEST, message)

        return True

    def mdf_chnpref(self, dump):
        """Изменить параметры глобальных параметров"""
        logging.debug("Change global preferences")
        # MDF_CHN_PREF

        message = '{"options":' + dump + '}'

        if self._control._connected:
            self._control._client.net_request(NetCmd.MDF_CHN_PREF, message)
        else:
            self._control.diffConfig.emit(NetCmd.MDF_CHN_PREF, message)

        return True

    def mdf_vkon(self, on=True, vk=None):
        """Включить/Выключить для тестирования указанный/все ВК
        :param on:
        :param vk:
        """
        # MDF_VK_ON     - on = True,  vk = {VK}
        # MDF_VK_OFF    - on = False, vk = {VK}
        # MDF_VKALL_ON  - on = True,  vk = None
        # MDF_VKALL_OFF - on = False, vk = None
        # TODO: MDF_VKALL_ON, MDF_VKALL_OFF

        message = '{"vkname":"' + vk + '"}'

        if on:
            if self._control._connected:
                self._control._client.net_request(NetCmd.MDF_VK_ON, message)
            else:
                self._control.diffConfig.emit(NetCmd.MDF_VK_ON, message)
        else:
            if self._control._connected:
                self._control._client.net_request(NetCmd.MDF_VK_OFF, message)
            else:
                self._control.diffConfig.emit(NetCmd.MDF_VK_OFF, message)

    def mdf_teston(self, on=True, test=None, vk=None):
        """Включить/Выключить для тестирования указанный/все тест(ы)
        :param on:
        :param test:
        """
        # TODO: MDF_TESTALL_ON, MDF_TESTALL_OFF
        # MDF_TEST_ON     - on = True,  test = {test},  vk = {VK}
        # MDF_TEST_OFF    - on = False, test = {test},  vk = {VK}
        # MDF_TESTALL_ON  - on = True,  test = None,    vk = None
        # MDF_TESTALL_OFF - on = False, test = None,    vk = None

        message = '{"testname":"' + test + '","vkname":"' + vk + '"}'

        if on:
            if self._control._connected:
                self._control._client.net_request(NetCmd.MDF_TEST_ON, message)
            else:
                self._control.diffConfig.emit(NetCmd.MDF_TEST_ON, message)
        else:
            if self._control._connected:
                self._control._client.net_request(NetCmd.MDF_TEST_OFF, message)
            else:
                self._control.diffConfig.emit(NetCmd.MDF_TEST_OFF, message)
