#!/usr/bin/env python
# -*- coding: utf-8 -*-


import json
import logging
import socket
import sys
from collections import OrderedDict
from copy import copy, deepcopy

from PyQt5 import QtGui, QtCore
from PyQt5.QtCore import Qt, QItemSelection, QItemSelectionModel
from PyQt5 import QtWidgets

import stp_gui.Common
from stp_gui.Clipboard import Clipboard
from stp_gui.Config import Config
from stp_gui.Config import test as Test
from stp_gui.Dialogs import AddTestDialog
from stp_gui.ExtraLogging import ExtraLog
from stp_gui.State import State


def _(text):
    return stp_gui.Common._(text)


class modificationsQueue:

    def __init__(self):
        # self.entries = []
        self.entries = {}

    def append(self, m):
        # self.entries.append(m)
        # self.entries[m.key] = m
        self.entries[(m.level, m.instance_num, m.key)] = m

    def clear(self):  # TODO: add couner to pop all unsaved or something like that
        self.entries.clear()

    def show(self):  # DBG
        for i in self.entries.values():
            print(i)
            print(80 * '-')
            print('key = ', (i.level, i.instance_num, i.key), 'value =', (i.value, i.index))
            print(80 * '+')


# ------------------------------------------------------------------------------
class modificationEntry:

    def __init__(self, key=None, value=None, level=None, instance_num=None, index=None):
        self.key = key
        self.value = value
        self.level = level  # 0 - root, 1 - host, 2 - test, 3 - instance, 4 - utest_opts
        self.instance_num = instance_num  # in which instnace was modification
        self.index = index


# ------------------------------------------------------------------------------
class treeStateRestorer(ExtraLog):

    def __init__(self):
        self.expanded_items = []

    def expanded(self, item):
        self.expanded_items.append(str(item.data(0, Qt.UserRole)))

    def collapsed(self, item):
        self.expanded_items.remove(str(item.data(0, Qt.UserRole)))

    def renamed(self, old_str, new_str):
        """Must be called if any node had been renamed"""
        self.debug('renaming %s as %s' % (old_str, new_str))
        add_list = []
        del_list = []
        for s in self.expanded_items:
            if s == old_str or s.startswith(old_str + '/'):
                add_list.append(s.replace(old_str, new_str, 1))
                del_list.append(s)
                # self.expanded_items.remove(s)
        l = [i for i in self.expanded_items if i not in del_list] + add_list
        self.expanded_items = l


# ------------------------------------------------------------------------------

class PropertiesWidget(ExtraLog, QtWidgets.QWidget):

    def __init__(self, mainwin):
        super(PropertiesWidget, self).__init__()

        self.mainWindow = mainwin
        # -----------------Queue of changes--------------------------------------
        self.mods = modificationsQueue()

        # -----------------OK and Cancel buttons---------------------------------
        self.buttons = QtWidgets.QDialogButtonBox()
        self.buttons.setStandardButtons(QtWidgets.QDialogButtonBox.Ok
                                        | QtWidgets.QDialogButtonBox.Cancel)
        self.buttons.button(QtWidgets.QDialogButtonBox.Ok).setText(_('OK'))
        self.buttons.button(QtWidgets.QDialogButtonBox.Cancel).setText(_('Cancel'))
        self.buttons.hide()
        self.buttons.accepted.connect(self.mainWindow.applyChanges)
        self.buttons.rejected.connect(self.mainWindow.rejectChanges)

        self.name_edit_text = ''

    def keyPressEvent(self, event):

        if (event.key() == Qt.Key_Enter or
                event.key() == Qt.Key_Return):
            self.buttons.accepted.emit()

        if event.key() == Qt.Key_Escape:
            self.buttons.rejected.emit()

        # super(PropertiesWidget, self).keyPressEvent(event)

    def showEmptyPanel(self):
        """
        Displays empty panel
        """
        for i in reversed(range(self.layout().itemAt(0).count())):
            l = self.layout().itemAt(0).itemAt(i).widget()
            if l is not None:
                l.setParent(None)
        self.mainWindow.tw.hide()

    def fillTableCells(self, propsDict, propSchema, level, delta=0, grid=None, instNum=None, hostname=None):
        """
        Displays settings for selected node
        """
        i = 0
        if delta == 0:
            # Clear layout by setting widget's parents to None
            for i in reversed(range(self.layout().itemAt(0).count())):
                l = self.layout().itemAt(0).itemAt(i).widget()
                if l is not None:
                    l.setParent(None)
        else:
            i = self.layout().itemAt(0).rowCount()
        s = self.sortProps(propsDict, propSchema)
        layout = self.layout().itemAt(0)
        if grid is not None:
            layout = grid
        # layout.setColumnStretch(1, 10)

        if hostname:
            name_label = QtWidgets.QLabel(_('Name'))
            name_edit = QtWidgets.QLineEdit(_('host'))
            name_edit.setText(hostname)

            def f(x):
                self.name_edit_text = x
                self.after_changes()

            name_edit.textChanged.connect(f)

            layout.addWidget(name_label, i, 0)
            layout.addWidget(name_edit, i, 1)
            i += 1
        else:
            name_edit = None
        for key, opt in s.items():
            # if key == 'testname':
            # continue
            if 'invisible' in propSchema[key]:
                if propSchema[key]['invisible']:
                    continue
            try:
                if propSchema[key]['type'] == 'integer':
                    enum = propSchema[key].get('enum', None)
                    alias = propSchema[key].get('aliasEnum', None)
                    if enum and alias:
                        enumStr = [_(alias[str(x)]) for x in enum]
                        combo = QtWidgets.QComboBox()
                        combo.addItems(enumStr)  # combo.addItems(QtCore.QStringList(enumStr))
                        layout.addWidget(combo, i, 1)
                        if opt != -1:
                            combo.setCurrentIndex(enum.index(opt))
                            combo.setToolTip(_(propSchema[key]['description']))
                        else:
                            combo.setCurrentIndex(combo.findText('<none>'))
                            combo.setToolTip(_(propSchema[key]['description']))
                        combo.currentIndexChanged.connect(
                            lambda v, k=key, d=delta, e=enum: self.modifyCombo(int(v),
                                                                               k, level, instNum, e))
                    elif enum:
                        enumStr = [str(x) for x in enum]
                        combo = QtWidgets.QComboBox()
                        combo.addItems(QtCore.QStringList(enumStr))
                        combo.setToolTip(_(propSchema[key]['description']))
                        layout.addWidget(combo, i, 1)
                        combo.setCurrentIndex(opt)
                        combo.currentIndexChanged.connect(
                            lambda v, k=key, d=delta: self.modifyConf(int(v),
                                                                      k, level, instNum))

                    else:
                        maxVal, minVal = None, None
                        if 'maximum' in propSchema[key]:
                            maxVal = propSchema[key]['maximum']
                        if 'minimum' in propSchema[key]:
                            minVal = propSchema[key]['minimum']
                        if minVal is not None and maxVal is not None:
                            spin = QtWidgets.QSpinBox()
                            spin.setMaximum(int(maxVal))
                            spin.setMinimum(int(minVal))
                            spin.setValue(opt)
                            spin.setToolTip(_(propSchema[key]['description']))
                            layout.addWidget(spin, i, 1)
                            spin.valueChanged.connect(
                                lambda v, k=key, d=delta: self.modifyConf(
                                    int(v), k, level, instNum))
                        else:
                            edit = QtWidgets.QLineEdit(str(opt), self)
                            if opt == -1:
                                edit.setText('')
                            edit.setValidator(QtGui.QIntValidator())
                            edit.setToolTip(_(propSchema[key]['description']))
                            layout.addWidget(edit, i, 1)
                            edit.textChanged.connect(
                                lambda v, k=key, d=delta: self.modifyConf(
                                    int(v) if v else -1, k, level, instNum))
                elif propSchema[key]['type'] == 'string':
                    enum = propSchema[key].get('enum', None)
                    if enum:
                        custom = propSchema[key].get('customValue', None)
                        # print 'custom = ', custom, type(custom)
                        enumStr = [str(x) if x is not None else '<none>' for x in enum]
                        combo = QtWidgets.QComboBox()
                        if custom:
                            combo.setInsertPolicy(QtWidgets.QComboBox.InsertAtBottom)
                            combo.setEditable(True)
                        combo.addItems(enumStr)
                        combo.setToolTip(_(propSchema[key]['description']))
                        layout.addWidget(combo, i, 1)
                        opt_str = opt if opt else '<none>'
                        if custom:
                            if opt_str in enum:
                                combo.setCurrentIndex(enumStr.index(opt_str))
                            else:
                                combo.setEditText(opt_str)
                        else:
                            combo.setCurrentIndex(enumStr.index(opt_str))
                        combo.currentIndexChanged.connect(lambda v, k=key, d=delta, e=enumStr:
                                                          self.modifyCombo(v, k, level, instNum, e))
                        combo.editTextChanged.connect(lambda v, k=key, d=delta, e=enumStr:
                                                      self.modifyCombo(len(e), k, level, instNum, e + [str(v)]))
                    else:
                        edit = QtWidgets.QLineEdit(opt)
                        edit.setToolTip(_(propSchema[key]['description']))
                        layout.addWidget(edit, i, 1)
                        edit.textChanged.connect(lambda v, k=key, d=delta: self.modifyConf(str(v), k, level, instNum))
                elif propSchema[key]['type'] == 'array':
                    continue
                elif propSchema[key]['type'] == 'object':
                    label = QtWidgets.QLabel()
                    label.setSizePolicy(QtWidgets.QSizePolicy.Ignored, QtWidgets.QSizePolicy.Fixed)
                    text = _(propSchema[key]['description'])
                    text = "<b>" + text + "</b>"
                    label.setText(text)
                    label.setAlignment(Qt.AlignCenter)
                    label.setToolTip(_(propSchema[key]['description']))
                    layout.addWidget(label, i, 0, 1, 2)
                    continue
                desc = propSchema[key]['title']
                label = QtWidgets.QLabel(_(desc))
                label.setToolTip(_(propSchema[key]['description']))
                layout.addWidget(label, i, 0)
                empty = QtWidgets.QWidget()
                empty.setSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Preferred)
                layout.addWidget(empty, i + 1, 0)
                i += 1
            except KeyError as e:
                logging.warning(_('property was not declared in schema: ') +
                                str(e))

    def sortProps(self, propsDict, propSchema):
        """
        Try to sort properties by priority specified in schema
        Called while displaying properties for a node
        """
        ids = []
        for val in propSchema.values():
            ids.append(val['id'])
        try:
            # Sort by numbers between first letter and colon
            def keyFunc(x):
                try:
                    return int(x.split(':')[0][1:])
                except ValueError:
                    return sys.maxsize

            # s = sorted(ids, key=lambda x: int(x.split(':')[0][1:]))
            s = sorted(ids, key=keyFunc)

            # keep only names after colon

            def splitFunc(x):
                try:
                    return x.split(':')[1]
                except IndexError:
                    return x

            s = list(map(splitFunc, s))

            s = s + list(set(propsDict.keys()) - set(s))
            # Filter properties not present in config
            # s = filter(lambda x: x in propsDict, s)
            s = [x for x in s if x in propsDict]
            return OrderedDict([(i, propsDict[i]) for i in s])
        except ValueError as e:
            logging.error(_('incorrect format of id field: ') + str(e))
            return propsDict

    def modifyCombo(self, value, key, level, instNum, e):
        self.debug('changing property %s on level %s to value %s from %s. %s' %
                   (key, level, value, e, 'instance' + str(instNum) if instNum is not None else ''))
        m = modificationEntry(key, value, level, instNum, e)
        self.mods.append(m)
        self.after_changes()

    def modifyConf(self, value, key, level, instNum):
        """
        Saves unapplied changes to dict
        """
        self.debug('changing property %s on level %s to value %s. %s' %
                   (key, level, value, 'instance' + str(instNum) if instNum is not None else ''))
        m = modificationEntry(key, value, level, instNum)
        self.mods.append(m)
        self.after_changes()

    def after_changes(self):
        self.buttons.show()
        self.mainWindow.userModified = True
        self.mainWindow.cfgTree.setDisabled(True)
        self.mainWindow.toolbar.setDisabled(True)
        self.mainWindow.menuBar().setDisabled(True)


# ------------------------------------------------------------------------------
class TreeWidget(QtWidgets.QTreeWidget):
    statusUpdated = QtCore.pyqtSignal(dict)
    statusBeginUpdate = QtCore.pyqtSignal()
    statusEndUpdate = QtCore.pyqtSignal()
    openMonitoring = QtCore.pyqtSignal()
    closeMonitoring = QtCore.pyqtSignal()

    def __init__(self, mainwin):
        super(TreeWidget, self).__init__()
        self.clipboard = Clipboard()

        self.monitorView = False
        self.mainWindow = mainwin
        # Last state received in heartbeat
        self.__prev_state = None
        # ----------------UI settings-------------------------------------------
        self.setColumnCount(6)
        self.setHeaderLabels([_('Test'), _('Status'), _('Total'), _('Time'), _('Passes'), _('Errors')])
        self.header().setSectionResizeMode(QtWidgets.QHeaderView.ResizeToContents)
        self.setSelectionMode(QtWidgets.QAbstractItemView.ExtendedSelection)
        sel_model = TreeWidgetSelectionModel(self.model(), self)

        def f(x, y):
            '''
            Emit currentItemChanged signal when selection changed
            x - QModelIndex current
            y - QModelIndex previous
            '''
            sibling = x.sibling(x.row(), 0)
            selected = sibling.data(Qt.UserRole)

            sibling = y.sibling(y.row(), 0)
            unselected = sibling.data(Qt.UserRole)

            self.currentItemChanged.emit(self.getNodeByPath(selected),
                                         self.getNodeByPath(unselected))

        sel_model.currentRowChanged.connect(lambda x, y: f(x, y))

        sel_model.selectionChanged.connect(self.afterSelection)
        self.setSelectionModel(sel_model)

        # ----------------Restoring tree state-----------------------------------
        self.treeState = treeStateRestorer()
        self.itemExpanded.connect(self.treeState.expanded)
        self.itemCollapsed.connect(self.treeState.collapsed)

        # -----------------OK and Cancel buttons---------------------------------
        self.buttons = QtWidgets.QDialogButtonBox()
        self.buttons.setStandardButtons(QtWidgets.QDialogButtonBox.Ok
                                        | QtWidgets.QDialogButtonBox.Cancel)
        self.buttons.button(QtWidgets.QDialogButtonBox.Ok).setText(_('OK'))
        self.buttons.button(QtWidgets.QDialogButtonBox.Cancel).setText(_('Cancel'))
        self.buttons.hide()
        # self.buttons.accepted.connect(self.mainWindow.applyChanges)
        # self.buttons.rejected.connect(self.mainWindow.rejectChanges)
        self.itemChanged.connect(self.selectionSwitchHandler)
        self.nodes_dict = {}
        self.ignore_dict = {}
        self._init_actions_()
        self.winWidth = 0
        self.winHeight = 0

    def resizeEvent(self, event):
        self.winWidth = event.size().width()
        self.winHeight = event.size().height()
        QtWidgets.QMainWindow.resizeEvent(self, event)

    def _init_actions_(self):
        delAction = QtWidgets.QAction("Del", self)
        self.addAction(delAction)
        delAction.triggered.connect(self.delEntity)
        delAction.setShortcut('d')

        addAction = QtWidgets.QAction("Add", self)
        self.addAction(addAction)
        addAction.triggered.connect(self.addEntity)
        addAction.setShortcut('a')

    def openMonitorView(self):
        self.monitorView = True

    def closeMonitorView(self):
        self.monitorView = False

    def addEntity(self):
        try:
            s = str(self.currentItem().data(0, Qt.UserRole))
        except AttributeError:
            self.addVK()
            return
        level = len(s.split('/')) - 1 if s != '/' else 0
        if level == 2:
            self.addTest()
        else:
            self.addVK()

    def delEntity(self):
        s = str(self.currentItem().data(0, Qt.UserRole))
        if s.count('/') == 2:
            self.delTest()
        elif s.count('/') == 1 and s != '/':
            self.delVK()
        else:
            logging.error("GUI: " + _("Can't delete root node"))

    def selectionSwitchHandler(self, w, column):
        if column:
            return

        path = w.getPatch()
        if not path or path == '/' or path == 'None':
            return
        # print 'selectionSwitchHandler'
        self.itemChanged.disconnect(self.selectionSwitchHandler)

        if w.checkState(0) == Qt.Checked:
            new_check = 1
        else:
            new_check = 0

        p = path.split('/')[1:]
        cfg = self.mainWindow.stpCtrl.get_config()
        if len(p) == 1:  # host
            for h in cfg.hosts:
                if h.hostname == p[0]:
                    if h.options.enable == new_check or h.options.enable == -1:
                        self.itemChanged.connect(self.selectionSwitchHandler)
                        return
                    action = bool(new_check)
                    self.itemChanged.connect(self.selectionSwitchHandler)
                    self.mainWindow.stpCtrl.mdf_vkon(on=action, vk=p[0])
        else:  # test
            for h in cfg.hosts:
                if h.hostname == p[0]:
                    for t in h.tests:
                        if t.testname == p[1]:
                            # TODO: Check schema if -1
                            if t.enable == new_check or t.enable == -1:
                                self.itemChanged.connect(self.selectionSwitchHandler)
                                return
                            action = bool(new_check)
                            self.mainWindow.stpCtrl.mdf_teston(on=action,
                                                               test=p[1], vk=p[0])
        self.itemChanged.connect(self.selectionSwitchHandler)
        self.displayCfg()

    def getNodeByPath(self, path):
        x = self.nodes_dict.get(str(path))
        return self.nodes_dict.get(str(path))

    def showNodeByPath(self, path):
        it = QtWidgets.QTreeWidgetItemIterator(self)
        while it.value():
            s = str(it.value().data(0, Qt.UserRole))
            if s == path:
                break
            it += 1
        self.setCurrentItem(it.value())

    def displayCfg(self, newCfg=False):
        """
        Display Config tree
        Called when Create/Open new Config
        """
        self.itemChanged.disconnect(self.selectionSwitchHandler)
        self.nodes_dict = {}
        self.currentItemChanged.disconnect(self.mainWindow.showNode)
        currentItemPath = '/'
        currentItem = None
        if not newCfg:
            if not self.currentItem():
                # If we are here, we have renamed a node

                # TODO: maybe we can do better then jumping to root node e.g.
                # check if there are nodes on this level left and select one
                rootItem = TreeWidgetItem(self, ['/'])
                self.setCurrentItem(rootItem)

            currentItemPath = str(self.currentItem().data(0, Qt.UserRole))
        self.clear()
        cfg = self.mainWindow.stpCtrl.get_config()
        rootItem = TreeWidgetItem(self, ['/'])
        rootItem.setData(0, Qt.UserRole, '/')
        self.nodes_dict['/'] = rootItem
        if cfg is None:
            self.itemChanged.connect(self.selectionSwitchHandler)
            return False
        if not newCfg and '/' in self.treeState.expanded_items:
            self.itemExpanded.disconnect(self.treeState.expanded)
            rootItem.setExpanded(True)
            self.itemExpanded.connect(self.treeState.expanded)
        if not newCfg and currentItemPath == '/':
            currentItem = rootItem
        for host in cfg.hosts:
            hostItem = TreeWidgetItem(rootItem, [host.hostname])
            hostItem.setFlags(hostItem.flags() | Qt.ItemIsUserCheckable)

            if host.options.enable:
                hostItem.setCheckState(0, Qt.Checked)
            else:
                hostItem.setCheckState(0, Qt.Unchecked)

            hostItem.setData(0, Qt.UserRole, '/' + host.hostname)
            self.nodes_dict['/' + str(host.hostname)] = hostItem
            sz = QtCore.QSize(100, 20)
            hostItem.setSizeHint(0, sz)
            itemPath = str(hostItem.data(0, Qt.UserRole))  # .ToString
            if not newCfg and itemPath in self.treeState.expanded_items:
                self.itemExpanded.disconnect(self.treeState.expanded)
                hostItem.setExpanded(True)
                self.itemExpanded.connect(self.treeState.expanded)
            if not newCfg and itemPath == currentItemPath:
                currentItem = hostItem
            if not host.tests:
                continue
            for test in host.tests:
                testItem = TreeWidgetItem(hostItem, [test.testname])
                testItem.setFlags(testItem.flags() | Qt.ItemIsUserCheckable)
                if test.enable:
                    testItem.setCheckState(0, Qt.Checked)
                else:
                    testItem.setCheckState(0, Qt.Unchecked)
                testItem.setData(0, Qt.UserRole, '/' + host.hostname + '/' + test.testname)
                self.nodes_dict['/' + str(host.hostname) + '/' + str(test.testname)] = testItem
                sz = QtCore.QSize(100, 20)
                testItem.setSizeHint(0, sz)
                itemPath = str(testItem.data(0, Qt.UserRole))  # .toString()
                if not newCfg and itemPath in self.treeState.expanded_items:
                    self.itemExpanded.disconnect(self.treeState.expanded)
                    testItem.setExpanded(True)
                    self.itemExpanded.connect(self.treeState.expanded)
                if not newCfg and itemPath == currentItemPath:
                    currentItem = testItem
                if not test.instances:
                    continue
        if newCfg:
            self.setCurrentItem(rootItem)
        else:
            if currentItem:
                self.setCurrentItem(currentItem)
            else:
                self.setCurrentItem(rootItem)

        self.sortItems(0, Qt.AscendingOrder)
        self.mainWindow.showNode()
        self.itemChanged.connect(self.selectionSwitchHandler)
        self.currentItemChanged.connect(self.mainWindow.showNode)

    def contextMenuEvent(self, event):
        if event.reason() == event.Mouse:
            pos = event.globalPos()
            item = self.itemAt(event.pos())
            if item is None:
                item = self.currentItem()
        else:
            pos = None
            selection = self.selectedItems()
            if selection:
                item = selection[0]
            else:
                item = self.currentItem()
                if item is None:
                    item = self.invisibleRootItem().child(0)
            if item is not None:
                parent = item.parent()
                while parent is not None:
                    parent.setExpanded(True)
                    parent = parent.parent()
                itemrect = self.visualItemRect(item)
                portrect = self.viewport().rect()
                if not portrect.contains(itemrect.topLeft()):
                    self.scrollToItem(
                        item, QtWidgets.QTreeWidget.PositionAtCenter)
                    itemrect = self.visualItemRect(item)
                itemrect.setLeft(portrect.left())
                itemrect.setWidth(portrect.width())
                pos = self.mapToGlobal(itemrect.center())

        # disable mouse right click if read only mode
        if self.mainWindow.isReadOnlyMode():
            return

        if item is None:
            return
        if pos is not None:
            menu = QtWidgets.QMenu(self)
            label = QtWidgets.QLabel('<b>' + item.text(0) + '</b>')
            label.setAlignment(Qt.AlignCenter)
            action = QtWidgets.QWidgetAction(menu)
            action.setDefaultWidget(label)
            menu.addAction(action)

        #        print item
        s = str(item.data(0, Qt.UserRole))  # .toString()
        #        print s
        level = len(s.split('/')) - 1 if s != '/' else 0
        #        print 'level: ', level

        if level == 0:
            addVkAction = menu.addAction(_('Add VK'))
            addVkAction.triggered.connect(self.addVK)

            pasteNodeAction = menu.addAction(_('Paste'))
            pasteNodeAction.triggered.connect(self.pasteNode)
        elif level == 1:
            addTestAction = menu.addAction(_('Add test'))
            addTestAction.triggered.connect(self.addTest)

            addVkAction = menu.addAction(_('Add VK'))
            addVkAction.triggered.connect(self.addVK)

            delVkAction = menu.addAction(_('Delete VK'))
            delVkAction.triggered.connect(self.delVK)

            renameAction = menu.addAction(_("Rename"))
            renameAction.triggered.connect(self.renameNode)

            copyNodeAction = menu.addAction(_('Copy node'))
            copyNodeAction.triggered.connect(self.copyNode)

            pasteNodeAction = menu.addAction(_('Paste'))
            pasteNodeAction.triggered.connect(self.pasteNode)
        elif level == 2:
            addTestAction = menu.addAction(_('Add test'))
            addTestAction.triggered.connect(self.addTest)

            delTestAction = menu.addAction(_('Delete test'))
            delTestAction.triggered.connect(self.delTest)

            renameAction = menu.addAction(_("Rename"))
            renameAction.triggered.connect(self.renameNode)

            copyNodeAction = menu.addAction(_('Copy node'))
            copyNodeAction.triggered.connect(self.copyNode)

            pasteNodeAction = menu.addAction(_('Paste'))
            pasteNodeAction.triggered.connect(self.pasteNode)

        action = menu.exec_(pos)
        event.accept()

    def mousePressEvent(self, event):
        '''
        Ignore left button mouse clicks not on elements of tree
        '''
        if (event.button() == Qt.LeftButton and
                self.itemAt(event.pos()) is None):
            return
        super(TreeWidget, self).mousePressEvent(event)

    def keyPressEvent(self, event):
        '''
        Ignore ctrl modifier with this keys:
            Qt::Key_Home
            Qt::Key_End
            Qt::Key_Left
            Qt::Key_Up
            Qt::Key_Right
            Qt::Key_Down
            Qt::Key_PageUp
            Qt::Key_PageDown
        to get rid of changing active element without changing selection
        ----------------------------------------------------------------
        select siblings on Ctrl+A
        ----------------------------------------------------------------
        shift + movement key selection on one level
        '''
        if (event.modifiers() == Qt.ControlModifier and
                Qt.Key_Home <= event.key() <= Qt.Key_PageDown):
            modifiers = event.modifiers() ^ Qt.ControlModifier
            new_event = QtGui.QKeyEvent(event.type(), event.key(), modifiers,
                                        event.text(), event.isAutoRepeat(),
                                        event.count())
            super(TreeWidget, self).keyPressEvent(new_event)
        if (event.modifiers() == Qt.ControlModifier and
                event.key() == Qt.Key_A):
            s = str(self.currentItem().data(0, Qt.UserRole))
            level = len(s.split('/')) - 1 if s != '/' else 0
            if level == 0:
                return
            else:
                parent = self.getNodeByPath(s).parent()
                for i in range(parent.childCount()):
                    parent.child(i).setSelected(True)
        elif event.modifiers() == Qt.ShiftModifier:
            s = str(self.currentItem().data(0, Qt.UserRole))
            level = len(s.split('/')) - 1 if s != '/' else 0
            if level == 0:
                return

            parent = self.getNodeByPath(s).parent()
            children_num = parent.childCount()
            child_index = parent.indexOfChild(self.getNodeByPath(s))
            if event.key() == Qt.Key_Down:
                if child_index == children_num - 1:
                    return
                else:
                    node = parent.child(child_index + 1)
                    if node.isSelected():
                        node.setSelected(False)
                    else:
                        node.setSelected(True)
                    self.setCurrentItem(node)
            elif event.key() == Qt.Key_Up:
                if child_index == 0:
                    return
                else:
                    node = parent.child(child_index - 1)
                    if node.isSelected():
                        node.setSelected(False)
                    else:
                        node.setSelected(True)
                    self.setCurrentItem(node)

        else:
            super(TreeWidget, self).keyPressEvent(event)

    def addVK(self):
        if not self.mainWindow.stpCtrl.config_present():
            self.mainWindow.stpCtrl.config_create()
        cfg = self.mainWindow.stpCtrl.get_config()
        hosts = cfg.hosts

        name, ok = QtWidgets.QInputDialog.getText(self, _('Enter VK name:'),
                                                  _('VK name:'))
        names = [x.hostname for x in hosts]
        if name in names:
            logging.error("GUI: " + _("Host with this name already exists"))
            return
        if ok and name.__len__() > 0:
            if Config.closed:
                self.mainWindow.stpCtrl.config_create()
            self.mainWindow.stpCtrl.mdf_addvk(str(name))

    def addTest(self):
        cfg = self.mainWindow.stpCtrl.get_config()
        try:
            s = str(self.currentItem().data(0, Qt.UserRole))
        except AttributeError:
            return
        if s.count('/') == 2:
            hostname = s.split('/')[-2]
            hosts = cfg.hosts
            host = next((host for host in hosts if host.hostname == hostname), None)
            names = [x.testname for x in host.tests]

            schema_items = copy(self.mainWindow.schemaparser.testOptions['type']['enum'])
            schema_items.remove(None)
            items = map(str, schema_items)

            dialog = AddTestDialog()
            dialog.set_combo_items(items)
            dialog.exec_()
            ok = bool(dialog.result())
            name, item = dialog.get_result()

            # name, ok = QtGui.QInputDialog.getText(self, _('Enter test name:'),
            # _('test name:'))
            if not ok or not name:
                return
            elif name in names:
                logging.error("GUI: " + _("Test with this name already exists"))
                return
            t = Test()
            t.testname = str(name)
            t.type = str(item)
            self.mainWindow.stpCtrl.mdf_addtest(hostname, name, t.dump())
        elif s.count('/') == 1:
            hostname = s.split('/')[-1]
            hosts = cfg.hosts
            host = next((host for host in hosts if host.hostname == hostname), None)
            names = [x.testname for x in host.tests]

            schema_items = copy(self.mainWindow.schemaparser.testOptions['type']['enum'])
            schema_items.remove(None)
            items = map(str, schema_items)

            dialog = AddTestDialog()
            dialog.set_combo_items(items)
            dialog.exec_()
            ok = bool(dialog.result())
            name, item = dialog.get_result()

            # name, ok = QtGui.QInputDialog.getText(self, _('Enter test name:'),
            # _('test name:'))
            if not ok or not name:
                return
            elif name in names:
                logging.error(_("Test with this name already exists"))
                return
            t = Test()
            t.testname = str(name)
            t.type = str(item)
            self.mainWindow.stpCtrl.mdf_addtest(hostname, name, t.dump())
        else:
            logging.error("GUI: " + _("This node is not a test node"))

    def delVK(self):
        settings = QtCore.QSettings()
        if not settings.value('tree/no_confirm_del'):
            button = QtWidgets.QMessageBox.question(
                self, _('Delete VK'), _('Are you sure you want to delete selected hosts?'),
                QtWidgets.QMessageBox.Yes, QtWidgets.QMessageBox.No)
            if button == QtWidgets.QMessageBox.No:
                return

        hosts_to_del = []
        for host in self.selectedItems():
            s = str(host.data(0, Qt.UserRole))
            if s.count('/') == 1 and s != '/':
                hosts_to_del.append(s.split('/')[-1])
            else:
                logging.error("GUI: " + _("This node is not a host"))
        for host in hosts_to_del:
            try:
                del self.ignore_dict['/' + str(host)]
            except KeyError:
                pass
            self.mainWindow.stpCtrl.mdf_delvk(host)

    def delTest(self):
        settings = QtCore.QSettings()
        if not bool(settings.value('tree/no_confirm_del')):
            button = QtWidgets.QMessageBox.question(
                self, _('Delete test'), _('Are you sure you want to delete selected tests?'),
                QtWidgets.QMessageBox.Yes, QtWidgets.QMessageBox.No)
            if button == QtWidgets.QMessageBox.No:
                return

        tests_to_del = []
        for test in self.selectedItems():
            s = str(test.data(0, Qt.UserRole))
            if s.count('/') == 2:
                tests_to_del.append((s.split('/')[-2], s.split('/')[-1]))
            else:
                logging.error("GUI: " + _("This node is not a test node"))
        for test in tests_to_del:
            self.mainWindow.stpCtrl.mdf_deltest(test[0], test[1])

    def renameNode(self):
        if not hasattr(self.mainWindow, 'level'):
            return
        if self.mainWindow.level == 0:
            logging.error("GUI: " + _("Can't rename root node"))
            return
        elif self.mainWindow.level == 1:  # Rename Host
            name, ok = QtWidgets.QInputDialog.getText(self, _('Enter new name:'),
                                                      _('VK name:'))
            if not ok or not name:
                return
            s = str(self.currentItem().data(0, Qt.UserRole))
            hostname = s.split('/')[1]
            hosts = self.mainWindow.stpCtrl.get_config().data['hosts']

            if name in [h.hostname for h in hosts]:
                logging.error("GUI: " + _("Node {} already exists").format(name))
                return

            h = next((x for x in hosts if x.hostname == hostname), None)
            new_h = deepcopy(h)
            new_h.hostname = str(name)
            try:
                del self.ignore_dict['/' + str(hostname)]
            except KeyError:
                pass
            self.treeState.renamed(s, '/' + str(name))
            self.mainWindow.stpCtrl.mdf_chnvk(hostname, new_h.dump())
            # self.showNodeByPath('/'+str(name))

        elif self.mainWindow.level == 2:  # Rename Test
            name, ok = QtWidgets.QInputDialog.getText(self, _('Enter new name:'),
                                                      _('test name:'))
            if not ok or not name:
                return
            s = str(self.currentItem().data(0, Qt.UserRole))
            hostname = s.split('/')[1]
            testname = s.split('/')[2]

            hosts = self.mainWindow.stpCtrl.get_config().data['hosts']
            host = next((x for x in hosts if x.hostname == hostname), None)
            tests = host.tests

            if name in [t.testname for t in tests]:
                logging.error("GUI: " + _("Node {} already exists").format(name))
                return

            t = next((x for x in tests if x.testname == testname), None)
            new_t = deepcopy(t)
            new_t.testname = str(name)
            self.treeState.renamed(s, '/' + str(hostname) + '/' + str(name))
            self.mainWindow.stpCtrl.mdf_chntest(hostname, testname, new_t.dump())
            # self.showNodeByPath('/'+str(hostname)+'/'+str(name))

        elif self.mainWindow.level == 3:
            logging.error("GUI:" + _("Can't rename instance node"))
            return

    def copyNode(self):
        nodes = []
        level = 0
        for i in self.selectedItems():
            s = str(i.data(0, Qt.UserRole))
            level = len(s.split('/')) - 1 if s != '/' else 0
            if level == 1:
                for host in self.mainWindow.stpCtrl.get_config().hosts:
                    if host.hostname == s.split('/')[-1]:
                        nodes.append(host)
            elif level == 2:
                for host in self.mainWindow.stpCtrl.get_config().hosts:
                    if host.hostname == s.split('/')[1]:
                        for test in host.tests:
                            if test.testname == s.split('/')[-1]:
                                nodes.append(test)
                                break
                        break

        self.clipboard.copy(nodes, level)

    def pasteNode(self):
        (data, level) = self.clipboard.get_data(self)
        if data is None or level is None:
            logging.info(_('Clipboard is empty'))
            return
        if level == 1:
            for node in data:
                hostname = node.hostname
                self.mainWindow.stpCtrl.mdf_addvk(hostname, node.dump())
        if level == 2:
            path = str(self.currentItem().data(0, Qt.UserRole))
            hostname = path.split('/')[1]
            # host_path = '/' + host
            for test in data:
                testname = test.testname
                self.mainWindow.stpCtrl.mdf_addtest(hostname, testname,
                                                    test.dump())

    def resetTestsData(self):
        it = QtWidgets.QTreeWidgetItemIterator(self)
        while it.value():
            it.value().setText(1, '')
            it.value().setText(2, '')
            it.value().setText(3, '')
            it.value().setText(4, '')
            it.value().setText(5, '')
            it += 1

    def updateStatus(self, data):
        state = json.loads(str(data))['system']['state']
        self.setUpdatesEnabled(False)

        # 1 -- running
        # 2 -- pause
        # 16 -- stop
        if state == 1:
            # print 'update status: running'
            if self.mainWindow.current_state != self.mainWindow.old_state:
                self.mainWindow.buttonsUnlock()
            self.mainWindow.stpCtrl.to_running()
            self.mainWindow.current_state = 1
        elif state == 2:
            if self.mainWindow.current_state != self.mainWindow.old_state:
                self.mainWindow.buttonsUnlock()
            self.mainWindow.stpCtrl.to_paused()
            self.mainWindow.current_state = 2
        elif state == 16:
            if self.mainWindow.current_state != self.mainWindow.old_state:
                self.mainWindow.buttonsUnlock()
            self.mainWindow.stpCtrl.to_ready()
            self.mainWindow.current_state = 16

        excl_id = json.loads(str(data))['system']['id']
        if self.mainWindow._excl_id != excl_id:
            self.mainWindow._excl_id = excl_id
        self.mainWindow.statusBar().update()
        data = json.loads(str(data))

        root = self.invisibleRootItem().child(0)
        self.updateNodeTest(root, data['system'])
        if not root:
            self.setUpdatesEnabled(True)
            return

        state = data['system']['state']
        if self.__prev_state == 0x0001 and state == 0x0010:
            self.mainWindow.stpCtrl.running_to_ready()
        self.__prev_state = state

        if 'hosts' not in data:
            self.setUpdatesEnabled(True)
            return

        if self.monitorView:
            self.statusBeginUpdate.emit()
            for host in data.get('hosts', ''):
                self.statusUpdated.emit(host)
            self.statusEndUpdate.emit()

        if not root.isExpanded():
            self.setUpdatesEnabled(True)
            return

        for i in range(root.childCount()):
            vkc = root.child(i)

            # search host in data['hosts'][]
            for host in data.get('hosts', ''):
                if host['hostname'] == vkc.text(0):
                    break
            if not host:
                continue
            self.updateNodeVk(vkc, host)

            if 'tests' not in host:
                continue

            if not vkc.isExpanded():
                continue

            for j in range(vkc.childCount()):
                tstc = vkc.child(j)

                # search test in data['hosts']['hostname']['tests']
                for test in host['tests']:
                    if test['testname'] == tstc.text(0):
                        break
                if not test:
                    continue
                self.updateNodeTest(tstc, test)

        QtCore.QCoreApplication.processEvents()
        self.setUpdatesEnabled(True)

    def _printState(self, bit, path, mes):
        value = self.ignore_dict.get(path)
        if value is None:
            logging.info(mes)
            self.ignore_dict[path] = bit
        elif value & bit != bit:
            logging.info(mes)
            self.ignore_dict[path] = value + bit

    def updateNodeVk(self, node, data):
        '''
        for status message was received before node
        in tree had been created -- ignored
        '''
        if not node:
            return

        '''
        for node who out of the window -- ignored
        '''
        rect = self.visualItemRect(node)
        if rect.y() >= self.winHeight or rect.y() < 0:
            return

        state = int(data['state'])

        if not hasattr(node, 'state'):
            node.state = 0

        hostname = data['hostname']
        path = node.data(0, Qt.UserRole)

        st = State(state, '\n')

        text = ''
        tooltip_text = ''
        if node.state != state:
            text = st.getShortStatus()
            tooltip_text = st.getFullStatus()

            if state & 0x0020:
                self._printState(0x0020, path, _("Host {} is connected.").format(hostname))

            if state & 0x0040:
                self._printState(0x0040, path, _("Host {} is terminated. Disconnected.").format(hostname))

            if state & 0x0080:
                self._printState(0x0080, path, _("Host {} is unavailable. Disconnected.").format(hostname))

            if state & 0x0100:
                self._printState(0x0100, path, _("Error while connecting to host {}. Disconnected.").format(hostname))

            if state & 0x0200:
                self._printState(0x0200, path, _("Host {} is disconnected.").format(hostname))

            if state & 0x1000:
                self._printState(0x1000, path, _("Host {} is busy. Disconnected.").format(hostname))

            if state & 0x400000:
                self._printState(0x400000, path, _("Older version on host {}. Disconnected.").format(hostname))

        if len(text):
            node.setToolTip(1, tooltip_text)
            node.setText(1, text)

        node.state = state

        node.setText(2, str(data['loop']))
        node.setText(3, str(data['time']))
        node.setText(4, str(data['passes']))
        node.setText(5, str(data['errors']))

    def updateNodeTest(self, node, data):
        '''
        for status message was received before node
        in tree had been created -- ignored
        '''
        if not node:
            return

        '''
        for node who out of the window -- ignored
        '''
        rect = self.visualItemRect(node)
        if rect.y() >= self.winHeight or rect.y() < 0:
            return

        state = int(data['state'])

        if not hasattr(node, 'state'):
            node.state = 0

        st = State(state, '\n')

        text = ''
        tooltip_text = ''
        if node.state != state:
            text = st.getShortStatus()
            tooltip_text = st.getFullStatus()

        if len(text):
            node.setToolTip(1, tooltip_text)
            node.setText(1, text)

        node.state = state

        node.setText(2, str(data['loop']))
        node.setText(3, str(data['time']))
        node.setText(4, str(data['passes']))
        node.setText(5, str(data['errors']))

    def afterSelection(self, x, y):
        '''
        Keeps selected and active items in sync
        x - QItemSelection selected
        y - QItemSelection deselected
        '''
        last_sel = self.selectionModel().last_selection
        zz = set()
        for i in last_sel.indexes():
            sibling = i.sibling(i.row(), 0)
            zz.add(str(sibling.data(Qt.UserRole)))
        if len(zz) == 0:
            return
        zz_item = list(zz)[-1]
        # if zz_item is None:
        # # logging.warning(_('No items were selected'))
        # return

        level = len(zz_item.split('/')) - 1 if zz_item != '/' else 0
        current_urls = set()
        selected_urls = set()
        unselected_urls = set()
        for i in self.selectedItems():
            path = str(i.data(0, Qt.UserRole))
            current_urls.add(path)
        for i in x.indexes():
            sibling = i.sibling(i.row(), 0)
            selected_urls.add(str(sibling.data(Qt.UserRole)))
        for i in y.indexes():
            sibling = i.sibling(i.row(), 0)
            unselected_urls.add(str(sibling.data(Qt.UserRole)))

        if not self.currentItem():
            return
        path = str(self.currentItem().data(0, Qt.UserRole))
        if (len(current_urls) == 1) and next(iter(current_urls)) != path:
            self.setCurrentItem(self.getNodeByPath(current_urls.pop()))
        elif len(current_urls) == 0 and len(unselected_urls) == 1:
            item = self.getNodeByPath(next(iter(unselected_urls)))
            if item:
                item.setSelected(True)

        n = self.getNodeByPath(zz_item)
        if n:
            parent = n.parent()
        else:
            parent = None

        if len(current_urls & zz) != 0:
            for s in current_urls:
                s_level = len(s.split('/')) - 1 if s != '/' else 0
                if s_level != level:
                    node = self.getNodeByPath(s)
                    node.setSelected(False)
                else:
                    node = self.getNodeByPath(s)
                    if node.parent() != parent:
                        sel_model = self.selectionModel()
                        node.setSelected(False)
                        sel_model.selectionChanged.connect(self.afterSelection)


class statusbar(QtWidgets.QStatusBar):

    def __init__(self, mainwin, parent=None):
        self.mainWindow = mainwin
        QtWidgets.QTreeWidget.__init__(self, parent)
        self.labelChange = QtWidgets.QLabel("")
        self.labelChange.setToolTip(_('Indicates if config has changed'))
        self.labelKernel = QtWidgets.QLabel("")
        self.labelKernel.setToolTip(_('IP address and port of host '
                                      'running kernel'))
        self.hb = 0
        self.labelHeartbeat = QtWidgets.QLabel("")
        self.labelHeartbeat.setToolTip(_('Kernel Heartbeat'))
        self.labelMode = QtWidgets.QLabel("")   
        self.labelMode.setToolTip(_('Indicates work mode: exclusive/watch only/normal'))
        hostname = socket.gethostname()
        self.labelVK = QtWidgets.QLabel(hostname)
        self.labelVK.setToolTip(_('System hostname'))
        time = QtCore.QDateTime.currentDateTime()
        self.labelTime = QtWidgets.QLabel(time.toString('hh:mm:ss dd.MM.yyyy'))
        self.labelTime.setToolTip(_('System time'))  # TODO wrong translation
        self.labelVersion = QtWidgets.QLabel("")
        self.labelVersion.setToolTip(_('STP version'))
        self.addPermanentWidget(self.labelChange, 1)
        self.addPermanentWidget(self.labelKernel, 1)
        self.addPermanentWidget(self.labelHeartbeat, 1)
        self.addPermanentWidget(self.labelMode, 1)
        self.addPermanentWidget(self.labelVK, 1)
        self.addPermanentWidget(self.labelTime, 1)
        self.addPermanentWidget(self.labelVersion, 1)
        self.setSizeGripEnabled(False)

    def update_time(self):
        time = QtCore.QDateTime.currentDateTime()
        self.labelTime.setText(time.toString('hh:mm:ss dd.MM.yyyy'))

    def update_heartbeat(self):
        if self.hb == 0:
            self.labelHeartbeat.setText("|")
        elif self.hb == 1:
            self.labelHeartbeat.setText("/")
        elif self.hb == 2:
            self.labelHeartbeat.setText("-")
        elif self.hb == 3:
            self.labelHeartbeat.setText("\\")
        else:
            self.labelHeartbeat.setText("")
        self.hb += 1
        self.hb %= 4

    def update(self):
        control = self.mainWindow.stpCtrl
        hostname = self.mainWindow._hostname
        port = self.mainWindow._port

        if control.config_ischanged():
            self.labelChange.setText(_("CFG was changed"))
        else:
            self.labelChange.setText("")

        if control.connected:
            self.labelKernel.setText(hostname + ":" + port)
        else:
            self.labelKernel.setText("")
            self.labelHeartbeat.setText("")

        if control.exclusive:
            text = "<font color='red'>" + _("Exclusive mode") + "</font>"
            self.labelMode.setText(text)
            self.mainWindow.exclusiveModeAction.setText(_("Disable &exclusive mode"))
            self.mainWindow.userModified = True
        else:
            if self.mainWindow._excl_id == -1 or self.mainWindow.stpCtrl.stp_id == -1:
                self.labelMode.setText("")
            elif self.mainWindow._excl_id != self.mainWindow.stpCtrl.stp_id:
                text = "<font color='blue'>" + _("Watch only mode") + "</font>"
                self.labelMode.setText(text)
            self.mainWindow.exclusiveModeAction.setText(_("Enable &exclusive mode"))
            self.mainWindow.userModified = False

        version = stp_gui.Common.APP_VERSION
        if len(version.split()) > 1:
            self.labelVersion.setText("Version: " + version.split()[2])
        else:
            self.labelVersion.setText("Version: " + version)


class TreeWidgetSelectionModel(QItemSelectionModel):

    def setCurrentIndex(self, index, command):
        super(TreeWidgetSelectionModel, self).setCurrentIndex(index, command)

    def select(self, selection, command):
        '''
        Show empy panel if multiple items are selecte in tree
        '''
        self.last_selection = self.selection()
        selected_urls = []
        if isinstance(selection, QItemSelection):
            for i in selection.indexes():
                sibling = i.sibling(i.row(), 0)
                selected_urls.append(sibling.data(Qt.UserRole))
        elif isinstance(selection, QtCore.QModelIndex):
            sibling = selection.sibling(selection.row(), 0)
            selected_urls.append(sibling.data(Qt.UserRole))
        before = len(self.selectedRows(1))
        super(TreeWidgetSelectionModel, self).select(selection, command)
        after = len(self.selectedRows(1))
        if after > 1:
            self.parent().mainWindow.propsPage.showEmptyPanel()
        if before > 1 and after == 1:
            self.parent().mainWindow.showNode()


# import traceback

class ResultsWidget(QtWidgets.QTabWidget):
    def __init__(self, mainwin):
        super(ResultsWidget, self).__init__()
        self.widgets = []
        self.mainWindow = mainwin
        self.setTabShape(QtWidgets.QTabWidget.Triangular)
        self.setStyleSheet("QTabWidget::pane { border: 1px solid; }")

        self.text = ''
        self.cache = {}  # Here we keep entries (hostname, testname, instance): log_text
        self.currentChanged.connect(self.changedTab)
        self.timer = QtCore.QTimer()

        def f():
            # self.mainWindow.getLogs
            curr = self.mainWindow.cfgTree.currentItem()
            self.displayLogs(curr, False)

        self.timer.timeout.connect(f)
        self.timer.start(5000)

    def changedTab(self, index):
        if index == -1:
            return
        curr = self.mainWindow.cfgTree.currentItem()
        path = str(curr.data(0, Qt.UserRole))
        level = len(path.split('/')) - 1 if path != '/' else 0
        if level < 1:
            # todo: print info about STP or cfg
            return

        hostname = path.split('/')[1]
        if level == 1:
            self.mainWindow.getHostInfo(hostname)
        else:
            testname = path.split('/')[2]
            self.mainWindow.getTestLogs(hostname, testname, index)

    def printInfo(self, data):
        info = _('Info about OS:') + '\n'
        info += '%-10s   %s\n' % (_('sysname'), data.get('sysname'))
        info += '%-10s   %s\n' % (_('nodename'), data.get('nodename'))
        info += '%-10s   %s\n' % (_('release'), data.get('release'))
        info += '%-10s   %s\n' % (_('build'), data.get('version'))
        info += '%-10s   %s\n\n' % (_('machine'), data.get('machine'))

        info += _('Info about devices:') + '\n'
        memstr = '%-10s   %s\n' % (_('memory'), data.get('memory'))
        cpustr = '%-10s   \n' % (_('cpu'))
        hddstr = '%-10s   \n' % (_('storage'))
        netstr = '%-10s   \n' % (_('network'))
        try:
            for i in data['devinfo']['list']:
                if str(i['type']) == 'cpu':
                    cpustr += '   %3d     %s %s\n' % (int(i['device']), i['freq'], _('Mhz'))
                if str(i['type']) == 'mem':
                    memstr = '%-10s   %d %s\n' % (_('memory'), int(i['size']), _('Mb'))
                if str(i['type']) == 'hdd':
                    hddstr += '   %s, %7s %s\n' % (i['device'], i['size'], _('GiB'))
            yesno = (lambda T: _('yes') if T else _('no'))
            for i in data['net']:
                netstr += '   %5s, %s, %s: %s, %s: %s\n' % (
                    i['name'], i['mac'], _('active'), yesno(i['up']), _('connect'), yesno(i['connect']))
        except Exception:
            cpustr += None
            hddstr += None
            netstr += None

        info += memstr + cpustr + hddstr + netstr

        return info

    def displayLogs(self, curr, force=False):
        if self.parent().parent().currentIndex() == 0:
            return
        if not force and not self.isVisible():
            return
        if curr is None:
            return
        inst_number = 0
        path = str(curr.data(0, Qt.UserRole))
        level = len(path.split('/')) - 1 if path != '/' else 0
        # print path, level
        if level == 0:
            return

        if level == 1:
            hostname = path.split('/')[1]
            for w in self.widgets:
                w.setParent(None)
            w = QtWidgets.QTextEdit()
            document = w.document()
            font = document.defaultFont()
            font.setFamily('Courier New')
            document.setDefaultFont(font)
            if hostname in self.cache:
                # print 'cache'
                data = self.cache[hostname]
                text = self.printInfo(data)
                # print text
                w.setPlainText(text)
                # traceback.print_stack()
            # else:
            #    self.mainWindow.getHostInfo(hostname)
            w.setReadOnly(True)
            # w.lineWrapMode = QtGui.QTextEdit.NoWrap
            self.widgets.append(w)
            self.addTab(w, _('Info'))
            return

        if level == 2:
            # print self.parent().parent().isTabEnabled(1)
            cfg = self.mainWindow.stpCtrl.get_config()
            hostname = path.split('/')[1]
            testname = path.split('/')[2]
            hosts = cfg.hosts
            host = next(
                (host for host in hosts if host.hostname == hostname), None)
            tests = host.tests
            test = next(
                (test for test in tests if test.testname == testname), None)
            inst_number = len(test.instances)

        for w in self.widgets:
            w.setParent(None)
        for i in range(0, inst_number):
            w = QtWidgets.QTextEdit()
            document = w.document()
            font = document.defaultFont()
            font.setFamily('Courier New')
            document.setDefaultFont(font)
            # self.text = host.hostname + ":" + test.testname + ":" + 'instance' + str(i)
            key = (host.hostname, test.testname, i)
            if key in self.cache:
                msg = self.cache[key]
                data = json.loads(str(msg))
                if 'testlog' not in data:
                    continue
                text = data['testlog']
                w.setPlainText(text)
            # w.setPlainText(self.text)
            w.setReadOnly(True)
            w.lineWrapMode = QtWidgets.QTextEdit.NoWrap
            self.widgets.append(w)
            self.addTab(w, 'instance ' + str(i))

    def setPlainText(self, text):
        self.w.setPlainText(text)

    def showInfo(self, msg):
        curr = self.mainWindow.cfgTree.currentItem()
        path = str(curr.data(0, Qt.UserRole))
        if len(path.split('/')) != 2:
            return

        hostname = path.split('/')[1]
        data = json.loads(str(msg))
        if hostname not in data.get('vkname', ''):
            return

        if hostname not in self.cache:
            # print 'put into cache'
            self.cache[hostname] = data
        text = self.printInfo(data)
        if self.currentWidget():
            self.currentWidget().setPlainText(text)

    def showLog(self, msg):
        # traceback.print_stack()
        curr = self.mainWindow.cfgTree.currentItem()
        path = str(curr.data(0, Qt.UserRole))
        if len(path.split('/')) < 3:
            return

        data = json.loads(str(msg))

        key = (data['vkname'], data['testname'], data['instance'])
        self.cache[key] = msg

        hostname = path.split('/')[1]
        testname = path.split('/')[2]
        if (data['vkname'] == hostname and
                data['testname'] == testname and
                data['instance'] == self.currentIndex()):
            if 'testlog' in data:
                text = data['testlog']
                self.currentWidget().setPlainText(text)


class LogViewer(QtWidgets.QTableWidget):
    def __init__(self, debug=False):
        self.debug = debug
        super(LogViewer, self).__init__()
        self.setColumnCount(3)
        self.setHorizontalHeaderLabels([_('Time'), _('Level'), _('Message')])
        self.verticalHeader().setSectionResizeMode(QtWidgets.QHeaderView.Stretch)
        self.horizontalHeader().setSectionResizeMode(QtWidgets.QHeaderView.ResizeToContents)
        self.horizontalHeader().setStretchLastSection(True)
        self.verticalHeader().hide()
        self.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)

    def append(self, mes):
        time_item = QtWidgets.QTableWidgetItem(mes[0])
        level_item = QtWidgets.QTableWidgetItem(mes[1])
        text_item = QtWidgets.QTableWidgetItem(mes[2])

        mes_type = mes[1]

        if mes_type in ('DEBUG'):
            level_item.setForeground(QtGui.QColor('blue'))
            if (not self.debug):
                return
        elif mes_type in ('INFO'):
            level_item.setForeground(QtGui.QColor('green'))
        elif mes_type in ('WARNING'):
            level_item.setForeground(QtGui.QColor('yellow'))
        elif mes_type in ('ERROR', 'CRITICAL'):
            level_item.setForeground(QtGui.QColor('red'))

        self.setRowCount(self.rowCount() + 1)
        self.setItem(self.rowCount() - 1, 0, time_item)
        self.setItem(self.rowCount() - 1, 1, level_item)
        self.setItem(self.rowCount() - 1, 2, text_item)

        # self.scrollToItem(time_item)
        self.scrollToBottom()

    def contextMenuEvent(self, event):
        menu = QtWidgets.QMenu(self)

        clearAction = menu.addAction(_('Clear log'))
        clearAction.triggered.connect(self.clearLog)

        saveLogAction = menu.addAction(_('Save log as...'))
        saveLogAction.triggered.connect(self.saveLog)

        menu.exec_(event.globalPos())

    def clearLog(self):
        self.setRowCount(0)

    def saveLog(self):
        filename = QtWidgets.QFileDialog.getSaveFileName(
            self, _('Save file as...'),
            '', _('STP GUI Log Files (*.log);;All Files (*)'))
        if filename[0] == ""''"" or not filename:
            return

        f = open(filename[0], 'w')

        for i in range(0, self.rowCount()):
            line = self.item(i, 0).text() + ' ' + self.item(i, 1).text() + ' ' + self.item(i, 2).text() + '\n'
            # py_line = str(line.toUtf8(), encoding='UTF-8')
            f.write(str(line.encode('utf8')))


class TreeWidgetItem(QtWidgets.QTreeWidgetItem):

    def __init__(self, parent=None):
        QtWidgets.QTreeWidgetItem.__init__(self, parent)

    def __init__(self, parent, text):
        QtWidgets.QTreeWidgetItem.__init__(self, parent, text)

    def __lt__(self, otherItem):
        column = self.treeWidget().sortColumn()

        if self.text(column).split('.')[0] != otherItem.text(column).split('.')[0]:
            return QtWidgets.QTreeWidgetItem.__lt__(self, otherItem)

        if len(self.text(column).split('.')) == 1:
            return True
        if len(otherItem.text(column).split('.')) == 1:
            return False
        if len(self.text(column).split('.')) == len(otherItem.text(column).split('.')) == 2:
            try:
                our = int(self.text(column).split('.')[1])
                theirs = int(otherItem.text(column).split('.')[1])
                return our < theirs
            except:
                pass

        return QtWidgets.QTreeWidgetItem.__lt__(self, otherItem)

    def getPatch(self):
        path = str(self.data(0, Qt.UserRole))
        return path
