# -*- coding: utf-8 -*-

from PyQt4 import QtGui, QtCore

import script_worker
import res_settings


class TestSelection(QtGui.QWidget):
    stateChanged = QtCore.pyqtSignal(name='TestStateChanged')

    def __init__(self, settings):
        super(TestSelection, self).__init__()
        self.settings = settings
        self.set_items = set()

        self.title = QtGui.QGroupBox('Test selector')

        # runnings programm
        run_prg_list = res_settings.test_run_program
        self.run_prg_group = QtGui.QButtonGroup()
        self.run_prg_group.setExclusive(False)
        run_prg_box = QtGui.QVBoxLayout()

        for name in run_prg_list:
            self.settings.add_test_run_program(name)
            run_prg_item = QtGui.QCheckBox(name)
            run_prg_item.setChecked(True)
            run_prg_item.setSizePolicy(QtGui.QSizePolicy.Fixed,
                                       QtGui.QSizePolicy.Fixed)
            self.run_prg_group.addButton(run_prg_item)
            run_prg_box.addWidget(run_prg_item)

        self.run_prg_group.buttonClicked.connect(self.run_prg_handler)

        # selection mode
        self.test_sel_mode_list = res_settings.test_mode_name
        self.test_sel_mode = QtGui.QComboBox()
        self.test_sel_mode.addItems(QtCore.QStringList(self.test_sel_mode_list))
        self.test_sel_mode.currentIndexChanged.connect(self.selection_mod_handler)

        top_layout = QtGui.QHBoxLayout()
        top_layout.addWidget(self.test_sel_mode)
        top_layout.addLayout(run_prg_box)

        # test list
        self.test_list = QtGui.QListWidget()
        self.test_list.itemChanged.connect(self.test_list_handler)

        self.box = QtGui.QVBoxLayout()
        self.box.addLayout(top_layout)
        self.box.setContentsMargins(0, 0, 0, 0)
        self.title.setLayout(self.box)

        layout = QtGui.QVBoxLayout()
        layout.addWidget(self.title)
        self.setLayout(layout)

    def update_list_items(self):
        if self.test_sel_mode.currentText() == self.test_sel_mode_list[-2]:
            index = 0
        elif self.test_sel_mode.currentText() == self.test_sel_mode_list[-1]:
            index = 1
        else:
            return
        tmp_set = {x[index] for x in
                   script_worker.get_test_list(self.settings)}
        remove_set = self.set_items - tmp_set
        add_set = tmp_set - self.set_items

        for name in remove_set:
            for item in self.test_list.findItems(name,
                                                 QtCore.Qt.MatchFixedString |
                                                 QtCore.Qt.MatchCaseSensitive):
                self.test_list.takeItem(self.test_list.row(item))

        for name in add_set:
            item = QtGui.QListWidgetItem(name)
            item.setFlags(QtCore.Qt.ItemIsUserCheckable |
                          QtCore.Qt.ItemIsEnabled)
            item.setCheckState(QtCore.Qt.Checked)
            self.test_list.addItem(item)

        self.set_items = tmp_set
        self.settings.set_test_list(list(self.set_items))

    def add_test_list(self):
        self.box.addWidget(self.test_list)

    def remove_test_list(self):
        self.box.removeWidget(self.test_list)

    def selection_mod_handler(self, index):
        mode = str(self.test_sel_mode.currentText())
        self.settings.set_test_mode(mode)
        if mode == self.test_sel_mode_list[-2] or mode == self.test_sel_mode_list[-1]:
            self.add_test_list()
            self.update_list_items()
        else:
            self.remove_test_list()
        self.stateChanged.emit()

    def run_prg_handler(self, item):
        name = str(item.text())
        if item.checkState():
            self.settings.add_test_run_program(name)
        else:
            self.settings.remove_test_run_program(name)
        self.stateChanged.emit()

    def test_list_handler(self, item):
        if item.checkState() == QtCore.Qt.Checked:
            self.settings.add_test_to_list(str(item.text()))
        else:
            self.settings.remove_test_from_list(str(item.text()))
        self.stateChanged.emit()


__author__ = 'alexey'
