# -*- coding: utf-8 -*-

from PyQt4 import QtGui, QtCore
import script_worker
import res_settings


class VkSelection(QtGui.QWidget):
    stateChanged = QtCore.pyqtSignal(name='VKStateChanged')

    def __init__(self, settings):
        super(VkSelection, self).__init__()
        self.settings = settings

        self.title = QtGui.QGroupBox('VK selector')

        self.vk_sel_mode_list = res_settings.vk_mode_names
        self.list_items = set()

        self.vk_list = QtGui.QListWidget()
        self.vk_list.itemChanged.connect(self.vk_check_handler)

        self.vk_sel_mode = QtGui.QComboBox()
        self.vk_sel_mode.addItems(QtCore.QStringList(self.vk_sel_mode_list))
        self.vk_sel_mode.currentIndexChanged.connect(self.vk_mode_handler)

        self.box = QtGui.QVBoxLayout()
        self.box.addWidget(self.vk_sel_mode)
        self.box.setContentsMargins(0, 0, 0, 0)
        self.title.setLayout(self.box)

        layout = QtGui.QVBoxLayout()
        layout.addWidget(self.title)
        self.setLayout(layout)

    def update_list_items(self):
        if self.vk_sel_mode.currentText() == self.vk_sel_mode_list[-2]:
            index = 0
        elif self.vk_sel_mode.currentText() == self.vk_sel_mode_list[-1]:
            index = 1
        else:
            return
        tmp_set = {x[index] for x in
                   script_worker.get_vk_list(self.settings)}
        remove_set = self.list_items - tmp_set
        add_set = tmp_set - self.list_items

        for name in remove_set:
            for item in self.vk_list.findItems(name,
                                               QtCore.Qt.MatchFixedString |
                                               QtCore.Qt.MatchCaseSensitive):
                self.vk_list.takeItem(self.vk_list.row(item))

        for name in add_set:
            item = QtGui.QListWidgetItem(name)
            item.setFlags(QtCore.Qt.ItemIsUserCheckable |
                          QtCore.Qt.ItemIsEnabled)
            item.setCheckState(QtCore.Qt.Checked)
            self.vk_list.addItem(item)

        self.list_items = tmp_set
        self.settings.set_vk_list(list(self.list_items))

    def add_vk_list(self):
        self.box.addWidget(self.vk_list)

    def remove_vk_list(self):
        self.box.removeWidget(self.vk_list)

    def vk_check_handler(self, item):
        if item.checkState() == QtCore.Qt.Checked:
            self.settings.add_vk_to_list(str(item.text()))
        else:
            self.settings.remove_vk_from_list(str(item.text()))
        self.stateChanged.emit()

    def vk_mode_handler(self, index):
        mode = str(self.vk_sel_mode.currentText())
        self.settings.set_vk_mode(mode)
        if mode == self.vk_sel_mode_list[-2] or mode == self.vk_sel_mode_list[-1]:
            self.add_vk_list()
            self.update_list_items()
        else:
            self.remove_vk_list()
        self.stateChanged.emit()


__author__ = 'alexey'
